@@ -64,6 +64,7 @@ CClient::CClient()
 	m_bSSLMode         = FALSE;
 	m_nNickListHandler = 0;
 	m_nOpListHandler   = 0;
+	m_bUsedPassword    = FALSE;
 
 	// set reconnect parameter
 	m_eReconnectState      = ersNONE;
@@ -338,6 +339,8 @@ int CClient::HandleMessage( const CString & message )
 						s += "HubTopic ";
 						// No $Hellos
 						s += "NoHello ";
+						// No $GetINFO
+						s += "NoGetINFO ";
 
 						if ( s != "" )
 						{
@@ -407,7 +410,7 @@ int CClient::HandleMessage( const CString & message )
 				}
 				else if ( m_bHandleUserList == TRUE )
 				{
-					if ( (m_UserList.AppendUser(msg->m_sNick) == TRUE) && (!m_bHandshake) )
+					if ( (m_UserList.AppendUser(msg->m_sNick) == TRUE) && (!m_bHandshake) && (m_MessageSupports.m_bNoGetInfo == FALSE) )
 						SendGetInfo(msg->m_sNick,GetNick());
 #ifdef HAVE_SSL
 					m_pMessageSSL.JoinHub(this,msg->m_sNick);
@@ -466,6 +469,11 @@ int CClient::HandleMessage( const CString & message )
 
 					m_UserList.InitUserList(msg);
 					
+					if (m_MessageSupports.m_bNoGetInfo == TRUE)
+					{
+						break;
+					}
+					
 					for(nick=0;(nick=msg->m_NickList.Next(nick))!=0;)
 					{
 						SendGetInfo(*nick,GetNick());
@@ -765,6 +773,8 @@ int CClient::RequestNickList()
 int CClient::Disconnect( bool force )
 {
 	UpdateReconnect( ersDISABLED );
+	
+	SetUsedPassword(FALSE);
 
 	return CConnection::Disconnect(force);
 }
