@@ -37,6 +37,8 @@
 #define XML_ANTISPAM		"antispam"
 #define XML_SPEED		"speed"
 #define XML_DESCRIPTION_TAG	"descriptiontag"
+#define XML_EXTENDED_HUB_COUNT	"extendedhubcount"
+#define XML_SUPPRESSED_NICKS	"suppressednicks"
 #define XML_AWAY_PREFIX		"awayprefix"
 
 #define XML_EMAIL_ENABLED	"emailenabled"
@@ -45,6 +47,9 @@
 #define XML_LOGFILE		"logfile"
 #define XML_LOGFILEON		"logfileon"
 #define XML_LOGFILENAME		"logfilename"
+#define XML_LOGDOWNLOADS	"logdownloads"
+#define XML_LOGUPLOADS		"loguploads"
+#define XML_LOGDETAILS		"logdetails"
 
 #define XML_SECURITY		"security"
 #define XML_FLOODOPKICKMESSAGE	"floodopkickmessage"
@@ -156,6 +161,7 @@
 #define XML_RECREATE_SHARELIST_TIME	"recreatesharelisttime"
 
 #define XML_DISABLEHASHLIST		"disablehashlist"
+#define XML_COMPRESSEDTRANSFERS		"enablecompressedtransfers"
 
 #define XML_TRAFFIC_RX			"trafficrx"
 #define XML_TRAFFIC_TX			"traffictx"
@@ -164,6 +170,13 @@
 #define XML_TRAFFIC_CONTROL_RX		"trafficcontrolrx"
 #define XML_TRAFFIC_CONTROL_TX		"trafficcontroltx"
 
+#define XML_AUTOSEARCHINTERVAL		"autosearchinterval"
+#define XML_SMALLFILESIZE		"smallfilesize"
+
+#define XML_DONTSHAREDOTFILES		"dontsharedotfiles"
+
+#define XML_REMOTE_ENCODING		"remote_encoding"
+
 #ifdef WIN32
 #define DCGUI_CONFIG_PATH	"\\dc"
 #else
@@ -191,10 +204,15 @@
 #include <dclib/cdownloadqueue.h>
 #include <dclib/clistenmanager.h>
 
+#include <dclib/core/ciconv.h>
+
 #include "cconfig.h"
 
 /** default hub url list */
 const char * huburl_list[] = {
+	"http://dchublist.com/hublist.config.bz2",
+	"http://dreamland.gotdns.org/PublicHubList.config.bz2",
+	"http://hublist.awenet.info/PublicHubList.config.bz2",
 	"http://www.hublist.org/PublicHubList.config.bz2",
 //	"http://www.neo-modus.com/PublicHubList.config",
 //	"http://dcplusplus.sourceforge.net/PublicHubList.config.bz2",
@@ -248,7 +266,7 @@ CConfig::CConfig( CString configpath )
 	sNick          = "nick";
 	sSearchNick    = "nick_search";
 	sAwayMessage   = "";
-	sDescription   = "http://dcgui.berlios.de";
+	sDescription   = "http://wxdcgui.sourceforge.net";
 	sSpeed         = "28.8Kbps";
 	m_sHost        = "ppp0";
 	m_bExternalIP  = FALSE;	
@@ -290,6 +308,9 @@ CConfig::CConfig( CString configpath )
 	bAutoRecreateShareList = FALSE;
 
 	bLogFile = 0;
+	bLogFinishedDownloads = TRUE;
+	bLogFinishedUploads = TRUE;
+	bLogDetails = FALSE;
 	sLogFile = "";
 
 	pBookmarkHubList = new CStringList();
@@ -352,7 +373,19 @@ CConfig::CConfig( CString configpath )
 	m_nRecreateShareListTime = 0;
 	m_bCreateFile = FALSE;
 	m_bDisableHashList = FALSE;
+	m_bCompressedTransfers = TRUE;
 
+	/* The previous hard coded interval was 10 minutes */
+	m_nAutoSearchInterval = 600;
+	/* Previous valknut value was 16KiB, DC++ increased it to 64KiB */
+	m_nSmallFileSize = 65536;
+	
+	/** Default to false, share dot files (previous behaviour) */
+	m_bDontShareDotFiles = FALSE;
+	
+	/** Default to ISO-8859-1 (previous behaviour) */
+	m_sRemoteEncoding = "ISO-8859-1";
+	
 	SetInstance(this);
 }
 
@@ -479,12 +512,16 @@ int CConfig::SaveDCLib()
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_SPEED,       sSpeed );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_ANTISPAM, bAntiSpam );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DESCRIPTION_TAG, bDescriptionTag );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_EXTENDED_HUB_COUNT, m_bUseExtendedHubCount );
 
 	// logfile
 	node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_LOGFILE, 0 );
 
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_LOGFILEON, bLogFile );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_LOGFILENAME, sLogFile );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_LOGDOWNLOADS, bLogFinishedDownloads );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_LOGUPLOADS, bLogFinishedUploads );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_LOGDETAILS, bLogDetails );
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_FLOODOPKICKMESSAGE, sFloodOpKickMessage);
 	
 	// transfer
@@ -565,6 +602,11 @@ int CConfig::SaveDCLib()
 	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_RELOAD_HUBLIST_TIME, CString().setNum(m_nReloadHubListTime) );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_FORCEMOVEENABLED, bForceMoveEnabled );
 	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DISABLEHASHLIST, m_bDisableHashList );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_COMPRESSEDTRANSFERS, m_bCompressedTransfers );
+	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_AUTOSEARCHINTERVAL, CString().setNum(m_nAutoSearchInterval) );
+	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_SMALLFILESIZE, CString().setNum(m_nSmallFileSize) );
+	xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DONTSHAREDOTFILES, m_bDontShareDotFiles );
+	xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_REMOTE_ENCODING, m_sRemoteEncoding );
 
 	// search history
 	node = xmlNewChild( doc->children, 0, (const xmlChar*)XML_SEARCH_HISTORY, 0 );
@@ -579,7 +621,7 @@ int CConfig::SaveDCLib()
 	// save file
 	s = sConfigPath + DCLIB_CONFIG;
 
-	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -704,7 +746,7 @@ int CConfig::SaveDCPublicHub()
 	// save file
 	s = sConfigPath + DCHUB_CONFIG;
 
-	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -750,7 +792,7 @@ int CConfig::SaveDCBookHub()
 	// save file
 	s = sConfigPath + DCBOOKHUB_CONFIG;
 
-	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -1047,7 +1089,7 @@ int CConfig::SaveDCTra( CStringList * queue, CStringList * chunks )
 
 	s = sConfigPath + DCTRA_CONFIG;
 
-	if ( xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1 )
+	if ( SaveConfigXmlViaTemp(s,doc) == -1 )
 	{
 		err = -1;
 	}
@@ -1317,6 +1359,8 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					bDescriptionTag = xml->getBoolChild(n2);
 				else if ( xml_name2 == XML_SPEED )
 					sSpeed = xml->content(n2);
+				else if ( xml_name2 == XML_EXTENDED_HUB_COUNT )
+					m_bUseExtendedHubCount = xml->getBoolChild(n2);
 			}
 		}
 		/* parse logfile entry */
@@ -1330,6 +1374,12 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					sLogFile = xml->content(n2);
 				else if ( xml_name2 == XML_LOGFILEON )
 					bLogFile = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_LOGDOWNLOADS )
+					bLogFinishedDownloads = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_LOGUPLOADS )
+					bLogFinishedUploads = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_LOGDETAILS )
+					bLogDetails = xml->getBoolChild(n2);
 			}
 		}
 		else if ( xml_name1 == XML_SECURITY )
@@ -1466,6 +1516,16 @@ void CConfig::ParseDCLibConfig( xmlNodePtr node )
 					m_nReloadHubListTime = xml->content(n2).asINT();
 				else if ( xml_name2 == XML_DISABLEHASHLIST )
 					m_bDisableHashList = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_COMPRESSEDTRANSFERS )
+					m_bCompressedTransfers = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_AUTOSEARCHINTERVAL )
+					m_nAutoSearchInterval = xml->content(n2).asLONG();
+				else if ( xml_name2 == XML_SMALLFILESIZE )
+					m_nSmallFileSize = xml->content(n2).asULL();
+				else if ( xml_name2 == XML_DONTSHAREDOTFILES )
+					m_bDontShareDotFiles = xml->getBoolChild(n2);
+				else if ( xml_name2 == XML_REMOTE_ENCODING )
+					m_sRemoteEncoding = xml->content(n2);
 			}
 		}
 		/* parse server entrys */
@@ -1538,7 +1598,12 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 	CString s = "";
 	bool tag = FALSE;
 	bool prof = FALSE;
+	bool extHubCount = TRUE;
 	enum eClientMode mode;
+	long normalHubs = 0;
+	long regHubs = 0;
+	long opHubs = 0;
+	long totalHubs = 0;
 	
 	// check profile
 	if ( (hubname != "") || (hubhost != "") )
@@ -1562,10 +1627,12 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 	if ( prof == TRUE )
 	{
 		tag = pConfigHubProfile.m_bTag;
+		extHubCount = pConfigHubProfile.m_bExtHubCount;
 	}
 	else
 	{
 		tag = bDescriptionTag;
+		extHubCount = m_bUseExtendedHubCount;
 	}
 	
 	// set description
@@ -1609,14 +1676,73 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 		}
 
 		s += ",H:";
-
-		if ( CConnectionManager::Instance() && (CConnectionManager::Instance()->GetConnectedHubCount(TRUE) > 0) )
+		
+		if (extHubCount) // long H:3/2/1 DC++ format
 		{
-			s += CString().setNum(CConnectionManager::Instance()->GetConnectedHubCount(TRUE));
+			if ( CConnectionManager::Instance() && (CConnectionManager::Instance()->GetConnectedHubCount(FALSE) > 0))
+			{
+				totalHubs = CConnectionManager::Instance()->GetConnectedHubCount(FALSE);
+				opHubs = totalHubs - CConnectionManager::Instance()->GetConnectedHubCount(TRUE);
+				regHubs = CConnectionManager::Instance()->GetConnectedHubPasswordCount() - opHubs;
+				normalHubs = totalHubs - (opHubs + regHubs);
+				
+				/* printf("Debug GetConnectedHubCount(FALSE): %d\n", CConnectionManager::Instance()->GetConnectedHubCount(FALSE));
+				 * printf("Debug GetConnectedHubCount(TRUE): %d\n", CConnectionManager::Instance()->GetConnectedHubCount(TRUE));
+				 * printf("Debug GetConnectedHubPasswordCount(): %d\n", CConnectionManager::Instance()->GetConnectedHubPasswordCount());
+			 	* printf("Debug total hub count: %d\n", totalHubs);
+			 	* printf("Debug OP hub count: %d\n", opHubs);
+			 	* printf("Debug reg hub count: %d\n", regHubs);
+			 	* printf("Debug normal hub count: %d\n", normalHubs);
+			 	*/
+				
+				// extra sanity check
+				if (normalHubs < 0)
+				{
+					printf("Warning! normal user hub count < 0, setting to 0\n");
+					normalHubs = 0;
+				}
+				
+				if (regHubs < 0)
+				{
+					printf("Warning! registered user hub count < 0, setting to 0\n");
+					regHubs = 0;
+				}
+				
+				if (opHubs < 0)
+				{
+					printf("Warning! operator hub count < 0, setting to 0\n");
+					opHubs = 0;
+				}
+				
+				// cannot be on zero hubs!
+				// this sometimes happens when logging on to a hub
+				// and the tag hasn't updated yet)
+				if ( (normalHubs == 0) && (regHubs == 0) && (opHubs == 0) )
+				{
+					normalHubs = 1;
+				}
+				
+				s += CString().setNum(normalHubs);
+				s += "/";
+				s += CString().setNum(regHubs);
+				s += "/";
+				s += CString().setNum(opHubs);
+			}
+			else
+			{
+				s += "1/0/0";
+			}
 		}
-		else
+		else // short H:6 original valknut format
 		{
-			s += "1";
+			if ( CConnectionManager::Instance() && (CConnectionManager::Instance()->GetConnectedHubCount(TRUE) > 0) )
+			{
+				s += CString().setNum(CConnectionManager::Instance()->GetConnectedHubCount(TRUE));
+			}
+			else
+			{
+				s += "1";
+			}
 		}
 
 		s += ",S:";
@@ -1635,7 +1761,17 @@ CString CConfig::GetDescription( bool raw, CString hubname, CString hubhost )
 		if ( lMaxUploadRate > 0 )
 		{
 			s += ",L:";
-			s += CString().setNum(lMaxUploadRate*1.0/1024,1);
+			
+			CString ratelimit = CString().setNum(lMaxUploadRate*1.0/1024,1);
+			
+			// remove trailing zero
+			// in case decimal separator is ","
+			if (ratelimit.Right(1) == "0")
+			{
+				ratelimit = ratelimit.Left(ratelimit.Length() - 2);
+			}
+			
+			s += ratelimit;
 		}
 
 		s += ">";
@@ -1909,14 +2045,17 @@ bool CConfig::AddPublicHub( const CString & name, const CString & host, const CS
 	if ( badd == TRUE )
 	{
 		hubitem = new DCConfigHubItem();
+		CIconv * ciconv = new CIconv( m_sRemoteEncoding, "UTF-8" );
 
 		hubitem->m_nID	        = (++m_nPublicHubID);
-		hubitem->m_sName        = name;
-		hubitem->m_sHost        = thost;
-		hubitem->m_sDescription = description;
+		hubitem->m_sName        = ciconv->encode(name);
+		hubitem->m_sHost        = ciconv->encode(thost);
+		hubitem->m_sDescription = ciconv->encode(description);
 		hubitem->m_sUserCount   = CString().setNum(ucount);
 		pPublicHubList->Add(sname,hubitem);
 		pPublicHubList_HostIndex->Add(shost,new CString(name));
+		
+		delete ciconv;
 	}
 
 	Thread.UnLock();
@@ -2588,11 +2727,13 @@ bool CConfig::SaveHubProfile()
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_AUTO_CONNECT, pConfigHubProfile->m_bAutoConnect );
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_SSL, pConfigHubProfile->m_bSSL );
 		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_DESCRIPTION_TAG, pConfigHubProfile->m_bTag );
+		xml->xmlNewBoolChild( node, 0, (const xmlChar*)XML_EXTENDED_HUB_COUNT, pConfigHubProfile->m_bExtHubCount );
+		xml->xmlNewStringChild( node, 0, (const xmlChar*)XML_SUPPRESSED_NICKS, pConfigHubProfile->m_sSuppressedNicks );
 	}
 
 	s = sConfigPath + DCPROF_CONFIG;
 
-	if ( ! (xmlSaveFormatFileEnc(s.Data(),doc,"utf-8",1) == -1) )
+	if ( ! (SaveConfigXmlViaTemp(s,doc) == -1) )
 	{
 		err = TRUE;
 	}
@@ -2655,6 +2796,10 @@ bool CConfig::LoadHubProfile()
 								pConfigHubProfile->m_bComment = xml->getBoolChild(n2);
 							else if ( xml_name == XML_EMAIL_ENABLED )
 								pConfigHubProfile->m_bEMail = xml->getBoolChild(n2);
+							else if ( xml_name == XML_EXTENDED_HUB_COUNT )
+								pConfigHubProfile->m_bExtHubCount = xml->getBoolChild(n2);
+							else if ( xml_name == XML_SUPPRESSED_NICKS )
+								pConfigHubProfile->m_sSuppressedNicks = xml->content(n2);
 						}
 
 						if ( pConfigHubProfile->m_sName != "" )
@@ -2823,3 +2968,48 @@ CString CConfig::AliasToPath( CString file )
 
 	return "";
 }
+
+/** */
+int CConfig::SaveConfigXmlViaTemp(CString filename, xmlDocPtr doc)
+{
+	int ret1, ret2, ret3;
+	ret1 = xmlSaveFormatFileEnc( (filename+".tmp").Data(), doc, "utf-8", 1 );
+	
+	if (ret1 == -1)
+	{
+		printf("Error saving %s\n", (filename+".tmp").Data());
+		return -1;
+	}
+	else
+	{
+		// check that the file exists before trying to delete it
+		// this is so valknut will work when first run
+		CFile configfile;
+		if ( configfile.Open(filename, IO_READONLY) == TRUE )
+		{
+			configfile.Close();
+			ret2 = remove(filename.Data());
+			if (ret2 == -1)
+			{
+				perror(("Removing " + filename + " failed").Data());
+				return -1;
+			}
+		}
+		else
+		{
+			printf("No existing config file %s to remove\n", filename.Data());
+		}
+		
+		ret3 = rename( (filename+".tmp").Data(), filename.Data() );
+		if (ret3 == -1)
+		{
+			perror(("Renaming temp to " + filename + " failed").Data());
+			return -1;
+		}
+		else
+		{
+			// all operations successful
+			return ret1;
+		}
+	}
+}
