@@ -63,6 +63,8 @@ public:
 		m_bAutoConnect = FALSE;
 		m_bSSL         = FALSE;
 		m_bTag         = TRUE;
+		m_bExtHubCount = TRUE;
+		m_sSuppressedNicks = "Hub";
 	};
 	/** */
 	DCConfigHubProfile( DCConfigHubProfile * p )
@@ -77,6 +79,8 @@ public:
 		m_bAutoConnect = p->m_bAutoConnect;
 		m_bSSL         = p->m_bSSL;
 		m_bTag         = p->m_bTag;
+		m_bExtHubCount = p->m_bExtHubCount;
+		m_sSuppressedNicks = p->m_sSuppressedNicks;
 	};
 	/** */
 	virtual ~DCConfigHubProfile() {};
@@ -101,6 +105,10 @@ public:
 	bool m_bSSL;
 	/** tag */
 	bool m_bTag;
+	/** If true, use H:3/2/1, if false, use H:6 */
+	bool m_bExtHubCount;
+	/** Regexp of nicks to not open chat windows for */
+	CString m_sSuppressedNicks;
 };
 
 class DLL_EXPORT DCConfigHubItem : public CObject {
@@ -196,6 +204,12 @@ public:
 	int SaveDCLib();
 	/** */
 	void ParseDCLibConfig( xmlNodePtr node );
+	/** Saves the config file by first writing it to a temp file,
+	  * deleting the original, then renaming the temp file back to
+	  * the original name.
+	  * Returns the number of bytes written, or -1 in case of error.
+	  */
+	int SaveConfigXmlViaTemp(CString filename, xmlDocPtr doc);
 
 	/** */
 	int LoadDCTra( CStringList * queue, CStringList * chunks );
@@ -203,9 +217,9 @@ public:
 	int SaveDCTra( CStringList * queue, CStringList * chunks );
 
 	/** */
-	CString GetDatadir() const;
+	CString GetDCLibDataPath() const;
 	/** */
-	void SetDatadir( CString s );
+	void SetDCLibDataPath( CString s );
 
 	/** */
 	CString ParseVersion( CString s );
@@ -266,7 +280,7 @@ public:
 	void SetCreateFile( bool b );
 
 	/** */
-	eClientMode GetMode();
+	eClientMode GetMode( bool setting = FALSE );
 	/** */
 	void SetMode( eClientMode mode );
 	/** */
@@ -330,6 +344,18 @@ public:
 	bool GetLogFile() const;
 	/** */
 	void SetLogFile( bool logfile );
+	/** True if we should log finished downloads */
+	bool GetLogFinishedDownloads() const;
+	/** Set to true to log finished downloads */
+	void SetLogFinishedDownloads( bool logdl );
+	/** True if we should log finished uploads */
+	bool GetLogFinishedUploads() const;
+	/** Set to true to log finished uploads */
+	void SetLogFinishedUploads( bool logul );
+	/** True if we should log the details in the log tab */
+	bool GetLogDetails() const;
+	/** Set to true to log details in the log tab */
+	void SetLogDetails( bool logdet );
 	/** */
 	CString GetLogFileName() const;
 	/** */
@@ -393,7 +419,7 @@ public:
 	/** */
 	bool RemovePublicHub( CString name, CString host, CString description );
 	/** */
-	bool AddPublicHub( const CString name, const CString host, const CString description, const CString usercount );
+	bool AddPublicHub( const CString & name, const CString & host, const CString & description, const CString usercount );
 	/** */
 	void ClearPublicHubList();
 	/** */
@@ -494,6 +520,41 @@ public:
 	/** */
 	void SetTransferKey( CString s );
 
+	/** */
+	bool GetDisableHashList() const;
+	/** */
+	void SetDisableHashList( bool b );
+	
+	/** */
+	bool GetCompressedTransfers() const;
+	/** */
+	void SetCompressedTransfers( bool b );
+	
+	/** Get the time in seconds between auto searches */
+	long GetAutoSearchInterval() const;
+	/** Set the time, in seconds, between auto searches */
+	void SetAutoSearchInterval( long seconds );
+	
+	/** Get the maximum size of file to give MiniSlots for */
+	ulonglong GetSmallFileSize() const;
+	/** Set the maximum size of file to give MiniSlots for */
+	void SetSmallFileSize( ulonglong s );
+	
+	/** If true, use H:3/2/1, if false, use H:6 */
+	bool GetUseExtendedHubCount() const;
+	/** */
+	void SetUseExtendedHubCount( bool b );
+	
+	/** If true, do not share files/folders starting with "."*/
+	bool GetDontShareDotFiles() const;
+	/** */
+	void SetDontShareDotFiles( bool b );
+	
+	/** Get remote encoding setting */
+	CString GetRemoteEncoding() const;
+	/** Set remote encoding setting */
+	void SetRemoteEncoding( CString name );
+	
 	/** */
 	CString AliasToPath( CString file );
 
@@ -546,6 +607,12 @@ private:
 	/** */
 	bool bLogFile;
 	/** */
+	bool bLogFinishedDownloads;
+	/** */
+	bool bLogFinishedUploads;
+	/** */
+	bool bLogDetails;
+	/** */
 	bool bCheckPrivateAddressSpace;
 	/** */
 	CString sLogFile;
@@ -598,7 +665,7 @@ private:
 	/** */
 	int iDownloadQueueTime;
 	/** */
-	CString sDatadir;
+	CString m_sDCLibDataPath;
 	/** */
 	bool bDynamicUploadRate;
 	/** */
@@ -618,6 +685,20 @@ private:
 	int m_nRecreateShareListTime;
 	/** */
 	bool m_bCreateFile;
+	/** */
+	bool m_bDisableHashList;
+	/** True if compressed transfers (currently download only) are enabled */
+	bool m_bCompressedTransfers;
+	/** The time in seconds between auto searches */
+	long m_nAutoSearchInterval;
+	/** The maximum size of files for which MiniSlots are granted */
+	ulonglong m_nSmallFileSize;
+	/** If true, use H:3/2/1, if false, use H:6 */
+	bool m_bUseExtendedHubCount;
+	/** If true, don't share files and folders starting with "." */
+	bool m_bDontShareDotFiles;
+	/** The remote text encoding */
+	CString m_sRemoteEncoding;
 
 protected:
 	/** */
@@ -812,6 +893,24 @@ inline bool CConfig::GetLogFile() const
 inline void CConfig::SetLogFile( bool logfile )
 { Thread.Lock(); bLogFile = logfile; Thread.UnLock(); }
 /** */
+inline bool CConfig::GetLogFinishedDownloads() const
+{ return bLogFinishedDownloads; }
+/** */
+inline void CConfig::SetLogFinishedDownloads( bool logdl )
+{ Thread.Lock(); bLogFinishedDownloads = logdl; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetLogFinishedUploads() const
+{ return bLogFinishedUploads; }
+/** */
+inline void CConfig::SetLogFinishedUploads( bool logul )
+{ Thread.Lock(); bLogFinishedUploads = logul; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetLogDetails() const
+{ return bLogDetails; }
+/** */
+inline void CConfig::SetLogDetails( bool logdet )
+{ Thread.Lock(); bLogDetails = logdet; Thread.UnLock(); }
+/** */
 inline CString CConfig::GetLogFileName() const
 { return sLogFile; }
 /** */
@@ -825,11 +924,11 @@ inline bool CConfig::GetCheckPrivateAddressSpace() const
 inline void CConfig::SetCheckPrivateAddressSpace( bool b )
 { Thread.Lock(); bCheckPrivateAddressSpace = b; Thread.UnLock(); }
 /** */
-inline CString CConfig::GetDatadir() const
-{ return sDatadir; }
+inline CString CConfig::GetDCLibDataPath() const
+{ return m_sDCLibDataPath; }
 /** */
-inline void CConfig::SetDatadir( CString s )
-{ Thread.Lock(); sDatadir = s; Thread.UnLock(); }
+inline void CConfig::SetDCLibDataPath( CString s )
+{ Thread.Lock(); m_sDCLibDataPath = s; Thread.UnLock(); }
 /** */
 inline bool CConfig::GetChatSendOfflineMessages() const
 { return bChatSendOfflineMessages; }
@@ -896,5 +995,47 @@ inline bool CConfig::GetCreateFile() const
 /** */
 inline void CConfig::SetCreateFile( bool b )
 { Thread.Lock(); m_bCreateFile = b; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetDisableHashList() const
+{ return m_bDisableHashList; }
+/** */
+inline void CConfig::SetDisableHashList( bool b )
+{ Thread.Lock(); m_bDisableHashList = b; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetCompressedTransfers() const
+{ return m_bCompressedTransfers; }
+/** */
+inline void CConfig::SetCompressedTransfers( bool b )
+{ Thread.Lock(); m_bCompressedTransfers = b; Thread.UnLock(); }
+/** */
+inline long CConfig::GetAutoSearchInterval() const
+{ return m_nAutoSearchInterval; }
+/** */
+inline void CConfig::SetAutoSearchInterval( long seconds )
+{ Thread.Lock(); m_nAutoSearchInterval = seconds; Thread.UnLock(); }
+/** */
+inline ulonglong CConfig::GetSmallFileSize() const
+{ return m_nSmallFileSize; }
+/** */
+inline void CConfig::SetSmallFileSize( ulonglong s )
+{ Thread.Lock(); m_nSmallFileSize = s; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetUseExtendedHubCount() const
+{ return m_bUseExtendedHubCount; }
+/** */
+inline void CConfig::SetUseExtendedHubCount( bool b )
+{ Thread.Lock(); m_bUseExtendedHubCount = b; Thread.UnLock(); }
+/** */
+inline bool CConfig::GetDontShareDotFiles() const
+{ return m_bDontShareDotFiles; }
+/** */
+inline void CConfig::SetDontShareDotFiles( bool b )
+{ Thread.Lock(); m_bDontShareDotFiles = b; Thread.UnLock(); }
+/** */
+inline CString CConfig::GetRemoteEncoding() const
+{ return m_sRemoteEncoding; }
+/** */
+inline void CConfig::SetRemoteEncoding( CString name )
+{ Thread.Lock(); m_sRemoteEncoding = name; Thread.UnLock(); }
 
 #endif
