@@ -253,6 +253,38 @@ long CConnectionManager::GetConnectedHubCount( bool admin )
 	return i;
 }
 
+/** */
+long CConnectionManager::GetConnectedHubPasswordCount( )
+{
+	long i;
+	CClient * client;
+
+	if ( !m_pClientList )
+	{
+		return 0;
+	}
+
+	m_pClientList->Lock();
+
+	i = 0;
+	client = 0;
+
+	while( (client=m_pClientList->Next(client)) != 0 )
+	{
+		if ( (client->IsHandshake() == TRUE) )
+			continue;	
+	
+		if ( (client->GetUsedPassword() == FALSE) )
+			continue;
+
+		i++;
+	}
+
+	m_pClientList->UnLock();
+
+	return i;
+}
+
 /** */
 CClient * CConnectionManager::GetHub( CString hubname, CString hubhost )
 {
