@@ -36,14 +36,27 @@
 #include <dclib/core/cbase64.h>
 #include <dclib/dcobject.h>
 
+// needed for remote encoding setting
+#include <dclib/cconfig.h>
+
 #include "cdcproto.h"
 
 CDCProto::CDCProto()
 {
+	CString to = "ISO-8859-1";
+	
+	if ( CConfig::Instance() != 0 )
+	{
+		to = CConfig::Instance()->GetRemoteEncoding();
+	}
+	
+	m_pIconv = new CIconv( "UTF-8", to );
 }
 
 CDCProto::~CDCProto()
 {
+	delete m_pIconv;
+	m_pIconv = 0;
 }
 
 /** */
@@ -59,8 +72,8 @@ int CDCProto::SendChat( CString sNick, CString s )
 	msg = msg.Replace( "|", "&#124;");
 
 	t  = "<";
-	t += sNick +"> ";
-	t += msg + "|";
+	t += m_pIconv->encode(sNick) +"> ";
+	t += m_pIconv->encode(msg) + "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
 
@@ -100,7 +113,7 @@ int CDCProto::SendMyInfo( CString sNick, CString sComment, CString sConnectionTy
 	CString t;
 
 	t  = "$MyINFO $ALL ";
-	t += sNick + " " + sComment + "$";
+	t += m_pIconv->encode(sNick) + " " + m_pIconv->encode(sComment) + "$";
 	t += " $"; // ???
 	t += sConnectionType;
 
@@ -115,7 +128,7 @@ int CDCProto::SendMyInfo( CString sNick, CString sComment, CString sConnectionTy
 	}
 
 	t += "$";
-	t += sEMail + "$";
+	t += m_pIconv->encode(sEMail) + "$";
 	t += sShare + "$"; // share
 	t += "|";
 
@@ -154,7 +167,7 @@ int CDCProto::SendValidateNick( CString sNick )
 	CString t;
 
 	t  = "$ValidateNick ";
-	t += sNick;
+	t += m_pIconv->encode(sNick);
 	t += "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
@@ -175,7 +188,7 @@ int CDCProto::SendMyNick( CString sNick )
 	CBase64 Base64;
 
 	t  = "$MyNick ";
-	t += sNick;
+	t += m_pIconv->encode(sNick);
 	t += "|";
 
 	t += "$Lock ";
@@ -255,14 +268,15 @@ int CDCProto::SendPrivateMessage( CString sNick, CString sTo, CString sMsg, CStr
 	CString t,msg;
 	CEncrypt Encrypt;
 
-	t = "$To: " + sTo + " From: " + sNick + " $<";
+	t = "$To: " + m_pIconv->encode(sTo) + " From: " + m_pIconv->encode(sNick) + " $<";
 	if ( sFromNick == "" )
-		t += sNick;
+		t += m_pIconv->encode(sNick);
 	else
-		t += sFromNick;
+		t += m_pIconv->encode(sFromNick);
 
 	msg = sMsg.Replace( "$", "&#36;");
 	msg = msg.Replace( "|", "&#124;");
+	msg = m_pIconv->encode(msg);
 
 	t += "> " + Encrypt.Encode(msg) + "|";
 
@@ -325,7 +339,7 @@ int CDCProto::SendGet( CString file, ulonglong pos, ulonglong size )
 	CString t;
 
 	t  = "$Get ";
-	t += file;
+	t += m_pIconv->encode(file);
 	t += "$";
 	t += CString().setNum(pos);
 
@@ -354,8 +368,15 @@ int CDCProto::SendGetZBlock( CString file, ulonglong pos, ulonglong size )
 
 	t  = "$GetZBlock ";
 	t += CString().setNum(pos) + " ";
-	t += CString().setNum(size) + " ";
-	t += file;
+	if (size == (ulonglong)-1)
+	{
+		t += "-1 ";
+	}
+	else
+	{
+		t += CString().setNum(size) + " ";
+	}
+	t += m_pIconv->encode(file);
 	t += "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
@@ -365,6 +386,62 @@ int CDCProto::SendGetZBlock( CString file, ulonglong pos, ulonglong size )
 	return err;
 }
 
+/** Send "$UGetBlock startPos numBytes fileNameInUTF8|" */
+int CDCProto::SendUGetBlock( CString file, ulonglong pos, ulonglong size )
+{
+	int err;
+	CString t;
+	
+	DCProtoThread.Lock();
+	
+	t  = "$UGetBlock ";
+	t += CString().setNum(pos) + " ";
+	if (size == (ulonglong)-1)
+	{
+		t += "-1 ";
+	}
+	else
+	{
+		t += CString().setNum(size) + " ";
+	}
+	t += file;
+	t += "|";
+	
+	err = Write((const unsigned char*)t.Data(),t.Length());
+	
+	DCProtoThread.UnLock();
+	
+	return err;
+}
+
+/** Send "$UGetZBlock startPos numBytes fileNameInUTF8|" */
+int CDCProto::SendUGetZBlock( CString file, ulonglong pos, ulonglong size )
+{
+	int err;
+	CString t;
+	
+	DCProtoThread.Lock();
+	
+	t  = "$UGetZBlock ";
+	t += CString().setNum(pos) + " ";
+	if (size == (ulonglong)-1)
+	{
+		t += "-1 ";
+	}
+	else
+	{
+		t += CString().setNum(size) + " ";
+	}
+	t += file;
+	t += "|";
+	
+	err = Write((const unsigned char*)t.Data(),t.Length());
+	
+	DCProtoThread.UnLock();
+	
+	return err;
+}
+
 /** */
 int CDCProto::SendRevConnectToMe( CString sNick, CString sDstNick )
 {
@@ -374,9 +451,9 @@ int CDCProto::SendRevConnectToMe( CString sNick, CString sDstNick )
 	CString t;
 
 	t  = "$RevConnectToMe ";
-	t += sNick;
+	t += m_pIconv->encode(sNick);
 	t += " ";
-	t += sDstNick;
+	t += m_pIconv->encode(sDstNick);
 	t += "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
@@ -413,7 +490,7 @@ int CDCProto::SendError( CString message )
 	CString t;
 
 	t  = "$Error ";
-	t += message;
+	t += m_pIconv->encode(message);
 	t += "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
@@ -488,7 +565,7 @@ int CDCProto::SendConnectToMe( CString sDstNick, CString host )
 	CString t;
 
 	t  = "$ConnectToMe ";
-	t += sDstNick + " ";
+	t += m_pIconv->encode(sDstNick) + " ";
 	t += host;
 	t += "|";
 
@@ -560,7 +637,7 @@ int CDCProto::SendKick( CString nick )
 	CString t;
 
 	t  = "$Kick ";
-	t += nick;
+	t += m_pIconv->encode(nick);
 	t += "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
@@ -579,11 +656,11 @@ int CDCProto::SendOpForceMove( CString nick, CString host, CString message )
 	CString t;
 
 	t  = "$OpForceMove $Who:";
-	t += nick;
+	t += m_pIconv->encode(nick);
 	t += "$Where:";
 	t += host;
 	t += "$Msg:";
-	t += message;
+	t += m_pIconv->encode(message);
 	t += "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
@@ -619,7 +696,7 @@ int CDCProto::SendGetInfo( CString sNick, CString sMyNick )
 	int err;
 	CString t;
 
-	t = "$GetINFO " + sNick + " " + sMyNick + "|";
+	t = "$GetINFO " + m_pIconv->encode(sNick) + " " + m_pIconv->encode(sMyNick) + "|";
 
 	err = Write((const unsigned char*)t.Data(),t.Length());
 
@@ -655,7 +732,7 @@ int CDCProto::SendSearch( CMessageSearchFile * msg )
 
 	if ( msg->m_bLocal == TRUE )
 		s += "Hub:";
-	s += msg->m_sSource;
+	s += m_pIconv->encode(msg->m_sSource);
 	s += " ";
 
 	if ( msg->m_bSizeLimit == FALSE )
@@ -673,7 +750,7 @@ int CDCProto::SendSearch( CMessageSearchFile * msg )
 	else
 		s += CString().setNum(msg->m_nSize) + "?";
 	s += CString().setNum(msg->m_eFileType) + "?";
-	s += msg->m_sString.Replace(' ',"$") + "|";
+	s += m_pIconv->encode(msg->m_sString.Replace(' ',"$")) + "|";
 
 	// passive searches with multisearch
 	if ( msg->m_bLocal == TRUE )
@@ -689,3 +766,84 @@ int CDCProto::SendSearch( CMessageSearchFile * msg )
 
 	return err;
 }
+
+/** $Sending numBytes| */
+int CDCProto::SendSending( ulonglong length )
+{
+	DCProtoThread.Lock();
+	
+	int err;
+	CString t;
+	
+	t  = "$Sending ";
+	t += CString().setNum(length);
+	t += "|";
+	
+	err = Write((const unsigned char*)t.Data(),t.Length());
+	
+	DCProtoThread.UnLock();
+	
+	return err;
+}
+
+/** $ADCGET file TTH/PPUROLR2WSYTGPLCM3KV4V6LJC36SCTFQJFDJKA 0 1154 ZL1| */
+int CDCProto::SendADCGet( CString type, CString tth, ulonglong pos, long long size, bool zlib )
+{
+	DCProtoThread.Lock();
+	
+	int err;
+	CString t;
+	
+	t  = "$ADCGET ";
+	t += type;
+	t += " ";
+	t += tth;
+	t += " ";
+	t += CString().setNum(pos);
+	t += " ";
+	t += CString().setNum(size);
+	if (zlib)
+	{
+		t += " ZL1";
+	}
+	t += "|";
+	
+	//printf("Sending ADCGet: %s\n", t.Data());
+	
+	err = Write((const unsigned char*)t.Data(),t.Length());
+	
+	DCProtoThread.UnLock();
+	
+	return err;
+}
+
+/** $ADCSND file TTH/PPUROLR2WSYTGPLCM3KV4V6LJC36SCTFQJFDJKA 0 1154 ZL1|*/
+int CDCProto::SendADCSnd( CString type, CString tth, ulonglong pos, long long size, bool zlib )
+{
+	DCProtoThread.Lock();
+	
+	int err;
+	CString t;
+	
+	t  = "$ADCSND ";
+	t += type;
+	t += " ";
+	t += tth;
+	t += " ";
+	t += CString().setNum(pos);
+	t += " ";
+	t += CString().setNum(size);
+	if (zlib)
+	{
+		t += " ZL1";
+	}
+	t += "|";
+	
+	//printf("Sending ADCSnd: %s\n", t.Data());
+	
+	err = Write((const unsigned char*)t.Data(),t.Length());
+	
+	DCProtoThread.UnLock();
+	
+	return err;
+}
