@@ -25,6 +25,7 @@
 #include <dclib/dcos.h>
 #include <dclib/core/cconnection.h>
 #include <dclib/core/cthread.h>
+#include <dclib/core/ciconv.h>
 
 class CMessageSearchFile;
 
@@ -54,7 +55,11 @@ public:
 	/** send get */
 	int SendGet( CString file, ulonglong pos, ulonglong size = 0 );
 	/** send GetZBlock */
-	int SendGetZBlock( CString file, ulonglong pos, ulonglong size );
+	int SendGetZBlock( CString file, ulonglong pos, ulonglong size = -1 );
+	/** send UGetBlock */
+	int SendUGetBlock( CString file, ulonglong pos, ulonglong size = -1 );
+	/** send UGetZBlock */
+	int SendUGetZBlock( CString file, ulonglong pos, ulonglong size = -1 );
 	/** send direction */
 	int SendDirection( eDirection Direction, int level );
 	/** send private message */
@@ -91,10 +96,18 @@ public:
 	int SendSupports( CString s );
 	/** */
 	int SendSearch( CMessageSearchFile * msg );
+	/** $Sending numBytes| */
+	int SendSending( ulonglong length );
+	/** */
+	int SendADCGet( CString type, CString tth, ulonglong pos, long long size, bool zlib = false );
+	/** */
+	int SendADCSnd( CString type, CString tth, ulonglong pos, long long size, bool zlib = false );
 
 private:
 	/** */
 	CThread DCProtoThread;
+	/** CIconv object for doing text encoding */
+	CIconv * m_pIconv;
 };
 
 #endif
