@@ -39,9 +39,12 @@
 #include <dclib/core/cbz.h>
 #include <dclib/core/che3.h>
 #include <dclib/csearchindex.h>
-#include <dclib/hash/tigertree.h>
 #include <dclib/core/cbase32.h>
 
+#include <dclib/hash/Encoder.h>
+#include <dclib/hash/TigerHash.h>
+#include <dclib/hash/MerkleTree.h>
+
 #include "cfilemanager.h"
 
 class CFileType : public CObject {
@@ -63,16 +66,26 @@ typedef struct filetype {
 
 struct filetype FileType[] = {
 	// MP3 2
+	{ "APE",  eftMP3 },
 	{ "AIFF", eftMP3 },
 	{ "AU",   eftMP3 },
+	{ "FLA",  eftMP3 },
+	{ "FLAC", eftMP3 },
 	{ "MID",  eftMP3 },
+	{ "M4A",  eftMP3 },
+	{ "M4P",  eftMP3 },
+	{ "MPC",  eftMP3 },
 	{ "MP2",  eftMP3 },
 	{ "MP3",  eftMP3 },
 	{ "OGG",  eftMP3 },
 	{ "RA",   eftMP3 },
+	{ "SHN",  eftMP3 },
 	{ "WAV",  eftMP3 },
+	{ "WMA",  eftMP3 },
+	{ "WV",   eftMP3 },
 
 	// ARCHIVE 3
+	{ "7Z",  eftARCHIVE },
 	{ "ACE", eftARCHIVE },
 	{ "BZ2", eftARCHIVE },
 	{ "CAB", eftARCHIVE },
@@ -102,15 +115,25 @@ struct filetype FileType[] = {
 	{ "INI",   eftDOCUMENT },
 	{ "INF",   eftDOCUMENT },
 	{ "LOG",   eftDOCUMENT },
+	{ "NFO",   eftDOCUMENT },
+	{ "ODG",   eftDOCUMENT },
+	{ "ODP",   eftDOCUMENT },
+	{ "ODS",   eftDOCUMENT },
+	{ "ODT",   eftDOCUMENT },
 	{ "PHP",   eftDOCUMENT },
+	{ "PPT",   eftDOCUMENT },
 	{ "PS",    eftDOCUMENT },
 	{ "PDF",   eftDOCUMENT },
 	{ "SHTML", eftDOCUMENT },
+	{ "SXC",   eftDOCUMENT },
+	{ "SXD",   eftDOCUMENT },
+	{ "SXI",   eftDOCUMENT },
 	{ "SXW",   eftDOCUMENT },
 	{ "TXT",   eftDOCUMENT },
 	{ "RFT",   eftDOCUMENT },
 	{ "RDF",   eftDOCUMENT },
 	{ "XML",   eftDOCUMENT },
+	{ "XLS",   eftDOCUMENT },
 
 	// APPL 5
 	{ "BAT",  eftAPPLICATION },
@@ -146,6 +169,7 @@ struct filetype FileType[] = {
 	{ "PIC",  eftPICTURE },
 	{ "PNG",  eftPICTURE },
 	{ "PCX",  eftPICTURE },
+	{ "PSP",  eftPICTURE },
 	{ "TGA",  eftPICTURE },
 	{ "TIF",  eftPICTURE },
 	{ "XPM",  eftPICTURE },
@@ -155,14 +179,17 @@ struct filetype FileType[] = {
 	{ "ASF",   eftVIDEO },
 	{ "ASX",   eftVIDEO },
 	{ "DAT",   eftVIDEO },
+	{ "MKV",   eftVIDEO },
 	{ "MOV",   eftVIDEO },
 	{ "MOVIE", eftVIDEO },
 	{ "MPG",   eftVIDEO },
 	{ "MP4",   eftVIDEO },
 	{ "MPEG",  eftVIDEO },
 	{ "OGM",   eftVIDEO },
+	{ "PXP",   eftVIDEO },
 	{ "QT",    eftVIDEO },
 	{ "RM",    eftVIDEO },
+	{ "RMVB",  eftVIDEO },
 	{ "VIV",   eftVIDEO },
 	{ "WMV",   eftVIDEO },
 	{ "DIVX",  eftVIDEO },
@@ -374,7 +401,7 @@ CStringList * CFileManager::SearchHash( CString s )
 
 	ts = s.Mid(4,s.Length()-4);
 	
-	if ( base32.Decode(&dst,&ts) != TIGERSIZE )
+	if ( base32.Decode(&dst,&ts) != TigerTree::HASH_SIZE )
 		return 0;
 
 	return m_pSearchIndex->SearchHash(dst.Data());
@@ -487,6 +514,9 @@ bool CFileManager::CreateHashList()
 
 		m_pFileManagerInfo->m_nProgress = 0;
 		m_pFileManagerInfo->m_eFileManagerStatus = efmsCREATEHASHLIST;
+		
+		// load hash leaves
+		m_pSearchIndex->OpenHashLeaves();
 
 		Start();
 		
@@ -645,6 +675,12 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 		return;
 
 	path = dir.SimplePath(m_pShareFolder->m_sPath+DIRSEPARATOR+curr+DIRSEPARATOR+rel);
+	
+	if ( (CConfig::Instance()->GetDontShareDotFiles() == TRUE) && (rel.Left(1) == ".") )
+	{
+		DPRINTF( "Not sharing folder \"%s\" due to dot files setting.\n", path.Data() );
+		return;
+	}
 
 	dir.SetPath(path);
 
@@ -670,6 +706,12 @@ void CFileManager::ThreadCreateShareList( int depth, CString curr, CString rel,
 		fileinfo = 0;
 		while( (fileinfo=list.Next(fileinfo)) != 0 )
 		{
+			if ( (CConfig::Instance()->GetDontShareDotFiles() == TRUE) && (fileinfo->name.Left(1) == ".") )
+			{
+				DPRINTF( "Not sharing file \"%s\" due to dot files setting.\n", fileinfo->name.Data() );
+				continue;
+			}
+			
 			if ( Stopped() )
 				return;
 
@@ -877,8 +919,6 @@ void CFileManager::ThreadCreateHashList()
 	CString s,pa;
 	struct filebaseobject fbo;
 	ulonglong hbi;
-	struct tt_context tth;
-	unsigned char hash[TIGERSIZE];
 	int len;
 	CFile f;
 	CBase32 base32;
@@ -904,7 +944,7 @@ void CFileManager::ThreadCreateHashList()
 		if ( m_pSearchIndex->FindHashBaseIndex(&fbo,&hbi) )
 		{
 			if ( dclibVerbose() > 0 )
-				printf("hash found, no changes in file\n");
+				printf("hash found, no changes in file %s\n", pa.Data());
 			fbo.m_nHashIndex = hbi;
 			m_pSearchIndex->UpdateIndex(m_nFileBaseIndex,&fbo);
 		}
@@ -912,13 +952,17 @@ void CFileManager::ThreadCreateHashList()
 		{
 			if ( CConfig::Instance()->GetDisableHashList() == FALSE )
 			{
+				if ( dclibVerbose() > 0 )
+					printf("calculate new hash for %s\n", pa.Data());
+				
 				if ( f.Open( pa, IO_RAW | IO_READONLY ) == TRUE )
 				{
-					tt_init(&tth);
+					CFile fi;
+					TigerTree tth( max( TigerTree::calcBlockSize(fi.GetFileSize(pa),10) , 64*1024 ) );
 
 					while ( (len = f.Read( (char*)m_pHashMemory->Data(), 1024*1024)) > 0 )
 					{
-						tt_update(&tth,m_pHashMemory->Data(),len);
+						tth.update(m_pHashMemory->Data(),len);
 				
 						if ( Stopped() )
 							break;
@@ -928,8 +972,11 @@ void CFileManager::ThreadCreateHashList()
 			
 					if ( !Stopped() )
 					{
-						tt_digest(&tth,(unsigned char*)hash);
-						m_pSearchIndex->AddHashIndex(m_nFileBaseIndex,hash);
+						tth.finalize();
+						vector<u_int8_t> a = tth.getLeafData();
+						m_pSearchIndex->AddHashIndex(m_nFileBaseIndex,
+							tth.getRoot().data,
+							&a[0],a.size());
 					}
 					else
 					{
@@ -975,6 +1022,10 @@ void CFileManager::ThreadCreateHashList()
 		}
 
 		m_pSearchIndex->SaveIndex();
+		
+		// SaveIndex() closes and saves hash leaves
+		//m_pSearchIndex->CloseHashLeaves();
+		
 		m_pShareList->CreateList(m_pSearchIndex);
 		
 		m_pFileManagerInfo->m_eFileManagerStatus = efmsNONE;
@@ -1005,6 +1056,10 @@ void CFileManager::Thread( CObject * )
 			ThreadCreateHashList();
 			wait = 0;
 			break;
+		case efmsREBUILDLISTS:
+			ThreadRebuildLists();
+			wait = 0;
+			break;
 		default:
 			break;
 	}
@@ -1040,3 +1095,75 @@ int CFileManager::FileManagerCallback( CObject *, CObject * )
 
 	return 0;
 }
+
+/** */
+CString CFileManager::GetFileName( CString fboid )
+{
+	struct filebaseobject fbo;
+	CString filename;
+
+	if ( GetFileBaseObject(fboid,&fbo,filename) )
+		return filename;
+	return "";
+}
+
+/** */
+bool CFileManager::IsCreateShareListRunning()
+{
+	if (m_pFileManagerInfo->m_eFileManagerStatus == efmsNONE)
+	{
+		return FALSE;
+	}
+	else
+	{
+		return TRUE;
+	}
+}
+
+/** */
+CByteArray * CFileManager::GetHashLeaves( CString tth )
+{
+	return m_pSearchIndex->GetHashLeaves( tth );
+}
+
+/** */
+bool CFileManager::RebuildLists()
+{
+	bool err = FALSE;
+	
+	//DPRINTF("CFileManager::RebuildLists() called\n");
+	
+	if ( m_pFileManagerInfo->m_eFileManagerStatus != efmsNONE )
+	{
+		return err;
+	}
+	
+	Lock();
+	
+	m_pFileManagerInfo->m_eFileManagerStatus = efmsREBUILDLISTS;
+	
+	Start();
+	
+	err = TRUE;
+	
+	UnLock();
+	
+	return err;
+}
+
+/** */
+void CFileManager::ThreadRebuildLists()
+{
+	//DPRINTF("CFileManager::ThreadRebuildLists() called\n");
+	
+	// CSearchIndex has direct access to the byte arrays so do it there
+	m_pSearchIndex->RebuildLists();
+	
+	m_pFileManagerInfo->m_nProgress = 0;
+	m_pFileManagerInfo->m_eFileManagerStatus = efmsIDLE;
+	
+	// recreate search index (also redoes hash lists)
+	CreateSearchIndex();
+	
+	//DPRINTF("CFileManager::ThreadRebuildLists() finished\n");
+}
