@@ -24,6 +24,7 @@
 #include <dclib/chttp.h>
 #include <dclib/core/cbz.h>
 #include <dclib/cfilemanager.h>
+#include <dclib/cutils.h>
 
 #ifndef WIN32
 #include <unistd.h>
@@ -54,6 +55,8 @@ CHubListManager::CHubListManager()
 	m_pHubListData    = 0;
 
 	m_bGetHubListDone = FALSE;
+	
+	m_pXml = new CXml();
 
 	SetInstance(this);
 }
@@ -72,6 +75,12 @@ CHubListManager::~CHubListManager()
 		delete m_pCallback;
 		m_pCallback = 0;
 	}
+	
+	if (m_pXml != 0)
+	{
+		delete m_pXml;
+		m_pXml = 0;
+	}
 }
 
 /** thread callbackfunction */
@@ -166,7 +175,14 @@ int CHubListManager::HttpCallBack( CObject *, CObject * object )
 						{
 							if ( m_pHttp->GetUrl().Find(".xml.") != -1 )
 							{
-								printf("XML Hublist not supported\n");
+								if ( m_pXml->ParseMemory((const char*)out.Data(),out.Size()) == TRUE )
+								{
+									ParseXmlPublicHubList();
+								}
+								else
+								{
+									printf("Failed to parse XML hublist.\n");
+								}
 							}
 							else
 							{
@@ -181,8 +197,22 @@ int CHubListManager::HttpCallBack( CObject *, CObject * object )
 					}
 					else
 					{
-						m_pHubListData->Append(in.Data(),in.Size());
-						m_pHubListData->Append("\xD\xA",2);
+						if ( m_pHttp->GetUrl().Right(4) == ".xml" ) 
+						{
+							if ( m_pXml->ParseMemory((const char*)in.Data(),in.Size()) == TRUE )
+							{
+								ParseXmlPublicHubList();
+							}
+							else
+							{
+								printf("Failed to parse XML hublist.\n");
+							}
+						}
+						else
+						{
+							m_pHubListData->Append(in.Data(),in.Size());
+							m_pHubListData->Append("\xD\xA",2);
+						}
 					}
 				}
 
@@ -312,6 +342,9 @@ int CHubListManager::ParsePublicHubList( CObject * , CObject * )
 			// replace all spaces
 			s2 = s2.Replace(" ","");
 
+			// remove line ends
+			s1 = s1.Replace("\n", "");
+
 			CConfig::Instance()->AddPublicHub( s1, s2, s3, s4 );
 		}
 
@@ -353,3 +386,136 @@ bool CHubListManager::NextHubListUrl()
 
 	return res;
 }
+
+/** */
+int CHubListManager::ParseXmlPublicHubList()
+{
+	xmlNodePtr node, node2;
+	int count = 0;
+	
+	printf("Parse XML hub list...\n");
+	
+	for ( node=m_pXml->doc()->children; node != 0; node=node->next )
+	{
+		if ( m_pXml->name(node) == "Hublist" )
+		{
+			for ( node2 = node->children; node2 != 0; node2=node2->next )
+			{
+				if ( m_pXml->name(node2) == "Hubs" )
+				{
+					count += ParseXmlHubs(node2->children);
+				}
+			}
+		}
+	}
+	
+	printf("XML hublist: %d hubs\n", count);
+	
+	if (count > 0)
+	{
+		m_pHubListData->Append("\xD\xA",2);
+	}
+	
+	return 0;
+}
+
+/** */
+int CHubListManager::ParseXmlHubs( xmlNodePtr node )
+{
+	xmlNodePtr n1;
+	CList<CXmlColumn> * cols = new CList<CXmlColumn>;
+	int count = 0;
+	
+	for ( n1=node; n1 != 0; n1=n1->next )
+	{
+		if ( m_pXml->name(n1) == "Columns" )
+		{
+			ParseXmlColumns(n1->children, cols);
+		}
+		else if ( m_pXml->name(n1) == "Hub" )
+		{
+			ParseXmlHub(n1, cols);
+			count++;
+		}
+	}
+	
+	cols->Clear();
+	delete cols;
+	
+	return count;
+}
+
+/** */
+void CHubListManager::ParseXmlColumns( xmlNodePtr node, CList<CXmlColumn> * cols )
+{
+	xmlNodePtr n1;
+	
+	for ( n1=node; n1 != 0; n1=n1->next )
+	{
+		if ( m_pXml->name(n1) == "Column" )
+		{
+			CXmlColumn * col = new CXmlColumn();
+			col->m_sName = m_pXml->prop(n1,"Name");
+			col->m_sType = m_pXml->prop(n1,"Type");
+			cols->Add(col);
+		}
+	}
+}
+
+/** */
+void CHubListManager::ParseXmlHub( xmlNodePtr node, CList<CXmlColumn> * cols )
+{
+	CString name = "";
+	CString server = "";
+	CString desc = "";
+	CString extra = " (";
+	int user = 0;
+	CXmlColumn * col = 0;
+	CString line = "";
+	
+	while ( (col = cols->Next(col)) != 0 )
+	{
+		col->m_sValue = m_pXml->prop(node, col->m_sName);
+		
+		if (col->m_sType == "bytes")
+		{
+			col->m_sValue = CUtils::GetSizeString( col->m_sValue.asULL(), euAUTO );
+		}
+		
+		if (col->m_sName == "Name")
+		{
+			name = col->m_sValue;
+		}
+		else if (col->m_sName == "Address")
+		{
+			server = col->m_sValue;
+		}
+		else if (col->m_sName == "Description")
+		{
+			desc = col->m_sValue;
+		}
+		else if (col->m_sName == "Users")
+		{
+			user = col->m_sValue.asINT();
+		}
+		else if (col->m_sName == "Port")
+		{
+			if (server.Find(':') == -1)
+			{
+				server = server + ":" + col->m_sValue;
+			}
+		}
+		else
+		{
+			extra += col->m_sName + "=" + col->m_sValue + " ";
+		}
+		
+		//printf("Name=%s Type=%s Value=%s\n", col->m_sName.Data(), col->m_sType.Data(), col->m_sValue.Data());
+	}
+	
+	extra += ")";
+	
+	line = name + "|" + server + "|" + desc + extra + "|" + CString().setNum(user) + "|||||\n\r";
+	
+	m_pHubListData->Append(line.Data(),line.Length());
+}
