@@ -23,9 +23,11 @@
 #include <dclib/dcobject.h>
 #include <dclib/core/cbytearray.h>
 #include <dclib/core/cbase64.h>
-#include <dclib/core/cxml.h>
 #include <dclib/core/csocket.h>
 
+// needed for remote encoding setting
+#include <dclib/cconfig.h>
+
 #include "cmessagehandler.h"
 
 // parsed messages
@@ -83,12 +85,26 @@
 #define DC_S_MESSAGE_SENDING		"$Sending "
 #define DC_S_MESSAGE_FAILED		"$Failed "
 
+//ADCGet command and response
+#define DC_S_MESSAGE_ADCGET		"$ADCGET "
+#define DC_S_MESSAGE_ADCSND		"$ADCSND "
+
 CMessageHandler::CMessageHandler()
 {
+	CString from = "ISO-8859-1";
+	
+	if ( CConfig::Instance() != 0 )
+	{
+		from = CConfig::Instance()->GetRemoteEncoding();
+	}
+	
+	m_pIconv = new CIconv( from, "UTF-8" );
 }
 
 CMessageHandler::~CMessageHandler()
 {
+	delete m_pIconv;
+	m_pIconv = 0;
 }
 
 int CMessageHandler::GetContent( const CString sMessage, const CString * sData, CString & sContent )
@@ -401,6 +417,18 @@ eDCMessage CMessageHandler::Parse( const CString * sMessage, int & pointer, CObj
 
 			return DC_MESSAGE_ERROR;
 		}
+		else if ( GetContent(DC_S_MESSAGE_ADCGET,&t,sContent) != -1 )
+		{
+			Object = ParseADCGet(sContent);
+			
+			return DC_MESSAGE_ADCGET;
+		}
+		else if ( GetContent(DC_S_MESSAGE_ADCSND,&t,sContent) != -1 )
+		{
+			Object = ParseADCSnd(sContent);
+			
+			return DC_MESSAGE_ADCSND;
+		}
 		else
 		{
 			printf("!!!!!!!!!!!!!! unknown start ");
@@ -428,7 +456,7 @@ CObject * CMessageHandler::ParseLock( CString sContent )
 
 	int i,i1;
 
-	i = sContent.Find(' ');
+	i = sContent.Find(" Pk=");
 
 	if ( (msg = new CMessageLock()) != 0 )
 	{
@@ -491,6 +519,16 @@ CObject * CMessageHandler::ParseLock( CString sContent )
 		{
 			msg->m_eClientVersion = eucvDCGUI;
 		}
+		// microdc client
+		else if ( (i1=msg->m_sPK.Find("MICRODC")) == 0 )
+		{
+			msg->m_eClientVersion = eucvMICRODC;
+		}
+		// shakespeer client
+		else if ( (i1=msg->m_sPK.Find("ShakesPeer")) == 0 )
+		{
+			msg->m_eClientVersion = eucvSHAKESPEER;
+		}
 
 		if ( msg->m_sVersionString != "" )
 		{
@@ -522,7 +560,7 @@ CObject * CMessageHandler::ParseHello( CString sContent )
 
 	if ( (msg = new CMessageHello()) != 0 )
 	{
-		msg->m_sNick = sContent;
+		msg->m_sNick = m_pIconv->encode(sContent);
 	}
 
 	return msg;
@@ -535,7 +573,7 @@ CObject * CMessageHandler::ParseMyNick( CString sContent )
 
 	if ( (msg = new CMessageMyNick()) != 0 )
 	{
-		msg->m_sNick = sContent;
+		msg->m_sNick = m_pIconv->encode(sContent);
 	}
 
 	return msg;
@@ -555,17 +593,17 @@ CObject * CMessageHandler::ParseChat( CString sContent )
 	{
 		if ( (i != -1) && (i1 != -1) )
 		{
-			msg->m_sNick    = sContent.Mid(i+1,i1-1);
+			msg->m_sNick    = m_pIconv->encode(sContent.Mid(i+1,i1-1));
 			// check if after the '>' a space
 			if ( sContent.Data()[i1+1] == ' ' )
 				i1++;
 
-			msg->m_sMessage = sContent.Mid(i1+1,sContent.Length()-i1-1);
+			msg->m_sMessage = m_pIconv->encode(sContent.Mid(i1+1,sContent.Length()-i1-1));
 		}
 		else
 		{
 			// no < nick > found ... use complete message
-			msg->m_sMessage = sContent;
+			msg->m_sMessage = m_pIconv->encode(sContent);
 		}
 
 		// reconvert dcproto chars
@@ -608,8 +646,8 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 
 	if ( (msg = new CMessageMyInfo()) != 0 )
 	{
-		msg->m_sNick    = sContent.Mid(i +1,i1-i -1);
-		msg->m_sUnknown = sContent.Mid(i2+1,i3-i2-1);
+		msg->m_sNick    = m_pIconv->encode(sContent.Mid(i +1,i1-i -1));
+		msg->m_sUnknown = m_pIconv->encode(sContent.Mid(i2+1,i3-i2-1));
 
 		c = 1;
 
@@ -687,11 +725,39 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 			msg->m_eUserSpeed = eusWIRELESS;
 		else if ( s == "Microwave" )
 			msg->m_eUserSpeed = eusMICROWAVE;
+		else if ( s == "0.005" )
+			msg->m_eUserSpeed = eus0005;
+		else if ( s == "0.01" )
+			msg->m_eUserSpeed = eus001;
+		else if ( s == "0.02" )
+			msg->m_eUserSpeed = eus002;
+		else if ( s == "0.05" )
+			msg->m_eUserSpeed = eus005;
+		else if ( s == "0.1" )
+			msg->m_eUserSpeed = eus01;
+		else if ( s == "0.2" )
+			msg->m_eUserSpeed = eus02;
+		else if ( s == "0.5" )
+			msg->m_eUserSpeed = eus05;
+		else if ( s == "1" )
+			msg->m_eUserSpeed = eus1;
+		else if ( s == "2" )
+			msg->m_eUserSpeed = eus2;
+		else if ( s == "5" )
+			msg->m_eUserSpeed = eus5;
+		else if ( s == "10" )
+			msg->m_eUserSpeed = eus10;
+		else if ( s == "20" )
+			msg->m_eUserSpeed = eus20;
+		else if ( s == "50" )
+			msg->m_eUserSpeed = eus50;
+		else if ( s == "100" )
+			msg->m_eUserSpeed = eus100;
 		else
 			msg->m_eUserSpeed = eusUNKNOWN;
 
 		msg->m_sUserSpeed = s;
-		msg->m_sEMail = sContent.Mid(i4+1,i5-i4-1);
+		msg->m_sEMail = m_pIconv->encode(sContent.Mid(i4+1,i5-i4-1));
 
 		s = sContent.Mid(i5+1,i6-i5-1);
 		msg->m_nShared = s.asULL();
@@ -724,6 +790,10 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 			msg->m_eClientVersion = eucvIDC;
 		else if ( (i=s.FindRev("<PerlDC")) != -1 )
 			msg->m_eClientVersion = eucvPERLDC;
+		else if ( (i=s.FindRev("<microdc")) != -1 )
+			msg->m_eClientVersion = eucvMICRODC;
+		else if ( (i=s.FindRev("<SP")) != -1 )
+			msg->m_eClientVersion = eucvSHAKESPEER;
 		else if ( (i=s.FindRev("<")) != -1 )
 			msg->m_eClientVersion = eucvUNKNOWN;
 		else
@@ -734,7 +804,7 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 		// find version end tag
 		if ( (i!=-1) && ((i1=s.Find('>',i)) != -1) )
 		{
-			msg->m_sComment    = s.Left(i);
+			msg->m_sComment    = m_pIconv->encode(s.Left(i));
 			msg->m_sVerComment = s.Mid(i,i1-i+1);
 
 			// try to find some default tags for unknown clients
@@ -761,7 +831,7 @@ CObject * CMessageHandler::ParseMyInfo( CString sContent )
 		// reset values
 		if ( msg->m_eClientVersion == eucvNONE )
 		{
-			msg->m_sComment    = s;
+			msg->m_sComment    = m_pIconv->encode(s);
 			msg->m_sVerComment = "";
 		}
 		
@@ -792,7 +862,7 @@ CObject * CMessageHandler::ParseConnectToMe( CString sContent )
 
 	if ( (msg = new CMessageConnectToMe()) != 0 )
 	{
-		msg->m_sNick = sContent.Mid(0,i);
+		msg->m_sNick = m_pIconv->encode(sContent.Mid(0,i));
 		msg->m_sHost = sContent.Mid(i+1,i1-i-1);
 
 		s = sContent.Mid(i1+1,sContent.Length()-i1-1);
@@ -828,8 +898,8 @@ CObject * CMessageHandler::ParseRevConnectToMe( CString sContent )
 
 	if ( (msg = new CMessageRevConnectToMe()) != 0 )
 	{
-		msg->m_sDstNick = sContent.Mid(0,i);
-		msg->m_sNick    = sContent.Mid(i+1,sContent.Length()-i-1);
+		msg->m_sDstNick = m_pIconv->encode(sContent.Mid(0,i));
+		msg->m_sNick    = m_pIconv->encode(sContent.Mid(i+1,sContent.Length()-i-1));
 	}
 
 	return msg;
@@ -849,7 +919,7 @@ CObject * CMessageHandler::ParseNickList( CString sContent )
 	{
 		CString s;
 
-		s = sContent.Mid(i1,i-i1);
+		s = m_pIconv->encode(sContent.Mid(i1,i-i1));
 
 		msg->m_NickList.Add(new CString(s));
 
@@ -874,7 +944,7 @@ CObject * CMessageHandler::ParseOpList( CString sContent )
 	{
 		CString s;
 
-		s = sContent.Mid(i1,i-i1);
+		s = m_pIconv->encode(sContent.Mid(i1,i-i1));
 
 		msg->m_NickList.Add(new CString(s));
 
@@ -890,7 +960,7 @@ CObject * CMessageHandler::ParseQuit( CString sContent )
 {
 	CMessageQuit * msg= new CMessageQuit();
 
-	msg->m_sNick = sContent;
+	msg->m_sNick = m_pIconv->encode(sContent);
 
 	return msg;
 }
@@ -933,7 +1003,7 @@ CObject * CMessageHandler::ParseSearch( CString sContent )
 		if ( s1 == "Hub:" )
 		{
 			msg->m_bLocal  = TRUE;
-			msg->m_sSource = s.Mid(i5+1,s.Length()-i5-1);
+			msg->m_sSource = m_pIconv->encode(s.Mid(i5+1,s.Length()-i5-1));
 		}
 		else
 		{
@@ -1022,7 +1092,7 @@ CObject * CMessageHandler::ParseSearch( CString sContent )
 		}
 
 		// the search string
-		s = sContent.Mid(i4+1,sContent.Length()-i4-1);
+		s = m_pIconv->encode(sContent.Mid(i4+1,sContent.Length()-i4-1));
 
 		// check if ext proto
 		/*
@@ -1065,6 +1135,32 @@ CObject * CMessageHandler::ParseSearch( CString sContent )
 
 /** */
 CObject * CMessageHandler::ParseSearchResult( CString sContent )
+{
+	CMessageSearchResult *msg=0;
+	int i,i1;
+	
+	if ( (i=sContent.Find(0x05)) < 0 )
+		return 0;
+	
+	if ( (i1=sContent.Find(0x05,i+1)) < 0 )
+	{
+		msg = (CMessageSearchResult*) ParseSearchResultFolder(sContent);
+		msg->m_bFolder = true;
+	}
+	else
+	{
+		msg = (CMessageSearchResult*) ParseSearchResultFile(sContent);
+		msg->m_bFolder = false;
+	}
+	
+	return msg;
+}
+
+/** 
+ * $SR <source_nick> <file_name><0x05><file_size> <free_slots>/<total slots><0x05><hub_name> (<hub_ip:port>)|
+ *                  i           i1               i2                         i3
+ */
+CObject * CMessageHandler::ParseSearchResultFile( CString sContent )
 {
 	CMessageSearchResult * msg=0;
 	CString s;
@@ -1088,8 +1184,8 @@ CObject * CMessageHandler::ParseSearchResult( CString sContent )
 
 	if ( (msg = new CMessageSearchResult()) != 0 )
 	{
-		msg->m_sNick = sContent.Mid(0,i);
-		msg->m_sFile = sContent.Mid(i+1,i1-i-1);
+		msg->m_sNick = m_pIconv->encode(sContent.Mid(0,i));
+		msg->m_sFile = m_pIconv->encode(sContent.Mid(i+1,i1-i-1));
 		s = sContent.Mid(i1+1,i2-i1-1);
 		msg->m_nSize = s.asULL();
 
@@ -1116,7 +1212,7 @@ CObject * CMessageHandler::ParseSearchResult( CString sContent )
 				if ( (i2 = s.FindRev('('),i1-1) != -1 )
 				{
 					if ( i2 > 0 ) i2--; // remove space " ("
-					msg->m_sHubName = s.Mid(0,i2);
+					msg->m_sHubName = m_pIconv->encode(s.Mid(0,i2));
 					msg->m_sHubHost = s.Mid(i2+2,i1-i2-2);
 				}
 			}
@@ -1155,6 +1251,99 @@ CObject * CMessageHandler::ParseSearchResult( CString sContent )
 	return msg;
 }
 
+/**
+ * $SR <source_nick> <directory_name> <free_slots>/<total_slots><0x05><hub_name> (<hub_ip:port>)|
+ *                  s1               s2                         i1              
+ */
+CObject * CMessageHandler::ParseSearchResultFolder( CString sContent )
+{
+	CMessageSearchResult * msg=0;
+	CString s;
+	int s1,s2,i1,slash,openBracket,closeBracket;
+
+/*	for(i=0;i<sContent.Length();i++)
+		printf("%02X ",sContent.Data()[i]);
+	printf("\n");*/
+
+	if ( (s1=sContent.Find(' ')) < 0 )
+		return 0;
+
+	if ( (i1=sContent.Find(0x05,s1+1)) < 0 )
+		return 0;
+
+	if ( (s2=sContent.FindRev(' ',i1)) < 0 )
+		return 0;
+
+	if ( (msg = new CMessageSearchResult()) != 0 )
+	{
+		msg->m_sNick = m_pIconv->encode(sContent.Mid(0,s1));
+		msg->m_sFile = m_pIconv->encode(sContent.Mid(s1+1,s2-s1-1));
+		msg->m_nSize = 0;
+
+		// parse the user slots
+		s = sContent.Mid(s2+1,i1-s2-1);
+		msg->m_nFreeSlot = 0;
+		msg->m_nMaxSlot  = 0;
+
+		if ( (slash = s.Find('/')) != -1 )
+		{
+			msg->m_nFreeSlot = s.Mid(0,slash).asUINT();
+			msg->m_nMaxSlot  = s.Mid(slash+1,s.Length()-slash-1).asUINT();
+		}
+
+		msg->m_sHubName = "";
+		msg->m_sHubHost = "";
+
+		s = sContent.Mid(i1+1,sContent.Length()-i1-1);
+
+		if ( s != "" )
+		{
+			if ( (closeBracket = s.FindRev(')')) != -1 )
+			{
+				if ( (openBracket = s.FindRev('('),closeBracket-1) != -1 )
+				{
+					if ( openBracket > 0 ) openBracket--; // remove space " ("
+					msg->m_sHubName = m_pIconv->encode(s.Mid(0,openBracket));
+					msg->m_sHubHost = s.Mid(openBracket+2,closeBracket-openBracket-2);
+				}
+			}
+		}
+
+		// check negative ports
+		CString ip;
+		unsigned int port;
+		
+		CSocket::ParseHost( msg->m_sHubHost, ip, port );
+		
+		msg->m_sHubHost = ip;
+		
+		if ( port != 0 )
+			 msg->m_sHubHost += ":" + CString().setNum(port);
+
+		// parse special content in hubname
+		if ( msg->m_sHubName.Left(4) == "TTH:" )
+		{
+			// tiger tree hash
+			msg->m_sHash = msg->m_sHubName;
+			// set dummy hubname
+			msg->m_sHubName = msg->m_sHubHost;
+		}
+
+		/* printf("%s\n%s : %s : %l : %s : %s : %s : %l : %l\n",\
+			sContent.Data(),
+			msg->m_sNick.Data(),
+			msg->m_sFile.Data(),
+			msg->m_nSize,
+			msg->m_sHubName.Data(),
+			msg->m_sHubHost.Data(),
+			msg->m_sHash.Data(),
+			msg->m_nFreeSlot,
+			msg->m_nMaxSlot ); */
+	}
+
+	return msg;
+}
+
 /** */
 CObject * CMessageHandler::ParsePrivateChat( CString sContent )
 {
@@ -1178,8 +1367,8 @@ CObject * CMessageHandler::ParsePrivateChat( CString sContent )
 
 	if ( (msg = new CMessagePrivateChat()) != 0 )
 	{
-		msg->m_sDstNick = sContent.Mid(0,i);
-		msg->m_sSrcNick = sContent.Mid(i1+1,i3-1-i1-1);
+		msg->m_sDstNick = m_pIconv->encode(sContent.Mid(0,i));
+		msg->m_sSrcNick = m_pIconv->encode(sContent.Mid(i1+1,i3-1-i1-1));
 
 		s = Encrypt.Decode(sContent.Mid(i3+1,sContent.Length()-i3-1));
 
@@ -1190,14 +1379,14 @@ CObject * CMessageHandler::ParsePrivateChat( CString sContent )
 
 		if ( (i==-1) || (i1==-1) )
 		{
-			msg->m_sMessage = s;
+			msg->m_sMessage = m_pIconv->encode(s);
 		}
 		else
 		{
 			if ( s.Data()[i1+1] == ' ' )
 				i1++;
-			msg->m_sMessage      = s.Mid(i1+1,s.Length()-i1-1);
-			msg->m_sMultiSrcNick = s.Mid(i+1,i1-2);
+			msg->m_sMessage      = m_pIconv->encode(s.Mid(i1+1,s.Length()-i1-1));
+			msg->m_sMultiSrcNick = m_pIconv->encode(s.Mid(i+1,i1-2));
 		}
 
 		// reconvert dcproto chars
@@ -1319,7 +1508,7 @@ CObject * CMessageHandler::ParseGet( CString sContent )
 
 	if ( (msg = new CMessageGet()) != 0 )
 	{
-		msg->m_sFilename = sContent.Mid(0,i);
+		msg->m_sFilename = m_pIconv->encode(sContent.Mid(0,i));
 
 		s = sContent.Mid(i+1,sContent.Length()-i-1);
 
@@ -1354,7 +1543,8 @@ CObject * CMessageHandler::ParseUGetBlock( CString sContent )
 		msg->m_nPos  = sContent.Mid(0,i).asULL();
 		msg->m_nSize = sContent.Mid(i+1,i1-i).asULL();
 		s = sContent.Mid(i1+1,sContent.Length()-i1-1);
-		msg->m_sFilename = CXml().FromUtf8(s);
+		msg->m_sFilename = s;
+		msg->m_bUGet = true;
 		
 		// fix start position
 		msg->m_nPos++;
@@ -1377,7 +1567,7 @@ CObject * CMessageHandler::ParseError( CString sContent )
 
 	if ( (msg = new CMessageError()) != 0 )
 	{
-		msg->m_sError = sContent;
+		msg->m_sError = m_pIconv->encode(sContent);
 	}
 
 	return msg;
@@ -1390,7 +1580,7 @@ CObject * CMessageHandler::ParseHubName( CString sContent )
 
 	if ( (msg = new CMessageHubName()) != 0 )
 	{
-		msg->m_sHubName = sContent;
+		msg->m_sHubName = m_pIconv->encode(sContent);
 	}
 
 	return msg;
@@ -1409,8 +1599,8 @@ CObject * CMessageHandler::ParseGetInfo( CString sContent )
 
 	if ( (msg = new CMessageGetInfo()) != 0 )
 	{
-		msg->m_sDstNick = sContent.Mid(0,i);
-		msg->m_sSrcNick = sContent.Mid(i+1,sContent.Length()-i-1);
+		msg->m_sDstNick = m_pIconv->encode(sContent.Mid(0,i));
+		msg->m_sSrcNick = m_pIconv->encode(sContent.Mid(i+1,sContent.Length()-i-1));
 	}
 
 	return msg;
@@ -1429,7 +1619,7 @@ CObject * CMessageHandler::ParseLogedIn( CString sContent )
 			sContent = sContent.Mid(1,sContent.Length()-1);
 		}
 
-		msg->m_sNick = sContent;
+		msg->m_sNick = m_pIconv->encode(sContent);
 	}
 
 	return msg;
@@ -1637,7 +1827,7 @@ CObject * CMessageHandler::ParseHubTopic( CString sContent )
 
 	if ( (msg = new CMessageHubTopic()) != 0 )
 	{
-		msg->m_sTopic = sContent;
+		msg->m_sTopic = m_pIconv->encode(sContent);
 	}
 
 	return msg;
@@ -1676,3 +1866,89 @@ CObject * CMessageHandler::ParseSending( CString sContent )
 
 	return msg;
 }
+
+/** $ADCGET file TTH/PPUROLR2WSYTGPLCM3KV4V6LJC36SCTFQJFDJKA 0 1154 ZL1| */
+CObject * CMessageHandler::ParseADCGet( CString sContent )
+{
+	CMessageADCGet * msg = 0;
+	int i;
+	CString s;
+	
+	if ( (msg = new CMessageADCGet()) != 0 )
+	{
+		//printf("sContent=\"%s\"\n", sContent.Data());
+	
+		i = sContent.Find(' ');
+		msg->m_sType = sContent.Mid(0,i);
+		
+		s = sContent.Mid(i+1,sContent.Length()-i-1);
+		
+		if (s.Right(4).ToUpper() == " ZL1")
+		{
+			msg->m_bZlib = true;
+			s = s.Mid(0, s.Length() - 4);
+		}
+		else
+		{
+			msg->m_bZlib = false;
+		}
+		
+		i = s.FindRev(' ');
+		msg->m_nSize = s.Mid(i, s.Length() - i).asLONGLONG();
+		s = s.Mid(0, i);
+		
+		i = s.FindRev(' ');
+		msg->m_nPos = s.Mid(i, s.Length() - i).asULL();
+		s = s.Mid(0, i);
+		
+		msg->m_sTTH = s;
+		
+		//printf("ADCGET Type=%s TTH=%s Pos=%lld Size=%lld Zlib=%d\n", msg->m_sType.Data(),
+		//	msg->m_sTTH.Data(), msg->m_nPos, msg->m_nSize, msg->m_bZlib );	
+	}
+	
+	return msg;
+}
+
+/** $ADCSND file TTH/PPUROLR2WSYTGPLCM3KV4V6LJC36SCTFQJFDJKA 0 1154 ZL1| */
+CObject * CMessageHandler::ParseADCSnd( CString sContent )
+{
+	CMessageADCSnd * msg = 0;
+	int i;
+	CString s;
+	
+	if ( (msg = new CMessageADCSnd()) != 0 )
+	{
+		//printf("sContent=\"%s\"\n", sContent.Data());
+		
+		i = sContent.Find(' ');
+		msg->m_sType = sContent.Mid(0,i);
+		
+		s = sContent.Mid(i+1,sContent.Length()-i-1);
+		
+		if (s.Right(4).ToUpper() == " ZL1")
+		{
+			msg->m_bZlib = true;
+			s = s.Mid(0, s.Length() - 4);
+		}
+		else
+		{
+			msg->m_bZlib = false;
+		}
+		
+		i = s.FindRev(' ');
+		msg->m_nSize = s.Mid(i, s.Length() - i).asLONGLONG();
+		s = s.Mid(0, i);
+		
+		i = s.FindRev(' ');
+		msg->m_nPos = s.Mid(i, s.Length() - i).asULL();
+		s = s.Mid(0, i);
+		
+		msg->m_sTTH = s;
+		
+		//printf("ADCSND Type=%s TTH=%s Pos=%lld Size=%lld Zlib=%d\n", msg->m_sType.Data(),
+		//	msg->m_sTTH.Data(), msg->m_nPos, msg->m_nSize, msg->m_bZlib );	
+	}
+	
+	return msg;
+}
