@@ -29,7 +29,7 @@
 
 #include "cbz.h"
 
-#define MAX_BUFFER_LEN	50000000	// 25mb max memory
+#define MAX_BUFFER_LEN	536870912	// 512MiB max memory
 
 /** */
 CBZ::CBZ()
@@ -82,7 +82,7 @@ bool CBZ::Compress( CByteArray * src, CByteArray * dst )
 			break;
 		}
 		
-		i = BZ2_bzBuffToBuffCompress( buffer, &outlen, (char*)src->Data(), inlen, 1, 0, 0);
+		i = BZ2_bzBuffToBuffCompress( buffer, &outlen, (char*)src->Data(), inlen, 9, 0, 0);
 
 		// check result
 		if ( i == BZ_OK )
@@ -122,7 +122,14 @@ bool CBZ::Decompress( CByteArray * src, CByteArray * dst )
 
 	while(TRUE)
 	{
-		outlen *= 10;
+		if ( outlen < (50 * 1024 * 1024) )
+		{
+			outlen *= 10;
+		}
+		else
+		{
+			outlen += (50 * 1024 * 1024);
+		}
 		
 		// check for max memory
 		if ( outlen > MAX_BUFFER_LEN )
