@@ -477,6 +477,20 @@ CString CString::setNum( const unsigned long n )
 	return CString(c);
 }
 
+/** */
+CString CString::setNum( const long long n )
+{
+	char c[50];
+
+#ifdef WIN32
+	_snprintf(c,50,"%lld",n);
+#else
+	snprintf(c,50,"%lld",n);
+#endif
+
+	return CString(c);
+}
+
 /** */
 CString CString::setNum( const ulonglong n )
 {
@@ -564,6 +578,23 @@ long CString::asLONG( int base ) const
 	return res;
 }
 
+long long CString::asLONGLONG( int base ) const
+{
+	long long res;
+	char *tail=0;
+	
+	if ( IsEmpty() )
+	{
+		return 0;
+	}
+	
+	res=strtoll(Data(),&tail,base);
+	
+	if((res==0)&&(errno!=0)) {}
+	
+	return res;
+}
+
 /** */
 double CString::asDOUBLE() const
 {
