@@ -23,17 +23,36 @@
 
 #include <stdlib.h>
 
+#include <dclib/core/ciconv.h>
+
 #include "cxml.h"
 
 /** */
 CXml::CXml()
 {
+	int i;
+	
 	pDoc = 0;
 
 	if ( xmlParserVersion != CString("20510") )
 	{
 		xmlInitParser();
 	}
+	
+	m_sLocaleEnc = getenv("LANG");
+	i = m_sLocaleEnc.Find('.');
+	if ( i == -1 )
+	{
+#ifdef __APPLE__
+		m_sLocaleEnc = "UTF-8";
+#else
+		m_sLocaleEnc = "";
+#endif
+	}
+	else
+	{
+		m_sLocaleEnc = m_sLocaleEnc.Mid(i + 1);
+	}
 }
 
 /** */
@@ -77,7 +96,7 @@ bool CXml::ParseFile( CString name )
 {
 	FreeDoc();
 
-	if ( (pDoc = xmlParseFile(name.Data())) == 0 )
+	if ( (pDoc = xmlRecoverFile(name.Data())) == 0 )
 	{
 		return FALSE;
 	}
@@ -90,7 +109,7 @@ bool CXml::ParseMemory( const char * s, int size )
 {
 	FreeDoc();
 
-	if ( (pDoc = xmlParseMemory(s,size)) == 0 )
+	if ( (pDoc = xmlRecoverMemory(s,size)) == 0 )
 	{
 		return FALSE;
 	}
@@ -129,91 +148,133 @@ CString CXml::prop( xmlNodePtr node, CString prop )
 }
 
 /** */
-CString CXml::xml_UTF8Toisolat1( char * s, int len )
+CString CXml::xml_UTF8ToOutput( char * s, int len, CString to_codec )
 {
 	CString r = "";
 	unsigned char *b;
 	int inlen,outlen,res;
+	CString codecname = "";
+	CIconv * ciconv = 0;
 
 	if ( (s == 0) || (len <= 0) )
 	{
 		return "";
 	}
 
-	inlen  = len;
-	outlen = inlen*4;
-
-	b = (unsigned char*) calloc(1,outlen);
-
-	if ( b == 0 )
-		return r;
-
-	res = UTF8Toisolat1( b, &outlen, (unsigned char*)s, &inlen );
-
-	if ( res == -2 )
-		printf("UTF8Toisolat1 transcoding fail: '%s'\n",s);
-	else if ( res == -1 )
-		printf("UTF8Toisolat1 fail: '%s'\n",s);
-	else if ( res >= 0 )
-		r = (char*)b;
+	if (to_codec == "")
+	{
+		codecname = m_sLocaleEnc;
+	}
 	else
-		printf("CXml::xml_UTF8Toisolat1 error %d\n",res);
+	{
+		codecname = to_codec;
+	}
 
-	free(b);
+	if ( codecname == "" )
+	{
+		inlen  = len;
+		outlen = inlen*4;
+
+		b = (unsigned char*) calloc(1,outlen);
+
+		if ( b == 0 )
+			return r;
+	
+		//printf("Warning: LANG does not specify codec or codec not found, using ISO-8859-1\n");
+		res = UTF8Toisolat1( b, &outlen, (unsigned char*)s, &inlen );
+		
+		if ( res == -2 )
+			printf("CXml::xml_UTF8ToOutput transcoding fail: '%s'\n",s);
+		else if ( res == -1 )
+			printf("CXml::xml_UTF8ToOutput fail: '%s'\n",s);
+		else if ( res >= 0 )
+			r = (char*)b;
+		else
+			printf("CXml::xml_UTF8ToOutput error %d\n",res);
+
+		free(b);
+	}
+	else
+	{
+		ciconv = new CIconv( "UTF-8", codecname );
+		r = ciconv->encode(s);
+		delete ciconv;
+	}
 
 	return r;
 }
 
 /** */
-CString CXml::xml_isolat1ToUTF8( char * s, int len )
+CString CXml::xml_InputToUTF8( char * s, int len, CString from_codec )
 {
 	CString r = "";
 	unsigned char *b;
 	int inlen,outlen,res;
+	CString codecname = "";
+	CIconv * ciconv = 0;
 
 	if ( (s == 0) || (len <= 0) )
 	{
 		return "";
 	}
 
-	inlen  = len;
-	outlen = inlen*4;
-
-	b = (unsigned char*) calloc(1,outlen);
-
-	if ( b == 0 )
-		return r;
-
-	res = isolat1ToUTF8( b, &outlen, (unsigned char*)s, &inlen );
-
-	if ( res >= 0 )
-		r = (char*)b;
+	if (from_codec == "")
+	{
+		codecname = m_sLocaleEnc;
+	}
 	else
-		printf("isolat1ToUTF8 fail: '%s'\n",s);
+	{
+		codecname = from_codec;
+	}
 
-	free(b);
+	if ( codecname == "" )
+	{
+		inlen  = len;
+		outlen = inlen*4;
+
+		b = (unsigned char*) calloc(1,outlen);
+
+		if ( b == 0 )
+			return r;
+		
+		//printf("Warning: LANG does not specify codec or codec not found, using ISO-8859-1\n");
+		res = isolat1ToUTF8( b, &outlen, (unsigned char*)s, &inlen );
+		
+		if ( res >= 0 )
+			r = (char*)b;
+		else
+			printf("CXml::xml_InputToUTF8 fail: '%s'\n",s);
+
+		free(b);
+	}
+	else
+	{
+		ciconv = new CIconv( codecname, "UTF-8" );
+		r = ciconv->encode(s);
+		delete ciconv;
+	}
 
 	return r;
 }
 
 /** */
-CString CXml::ToUTF8( const char * s )
+CString CXml::ToUTF8( const char * s, CString from_codec, bool nosub )
 {
 	CString str = s;
 
-	return ToUTF8(str);
+	return ToUTF8(str, from_codec, nosub);
 }
 
 /** */
-CString CXml::FromUtf8( const char * s )
+CString CXml::FromUtf8( const char * s, CString to_codec, bool nosub )
 {
 	CString str = s;
 
-	return FromUtf8(str);
+	return FromUtf8(str, to_codec, nosub);
 }
 
 /** */
-CString CXml::ToUTF8( CString & s )
+CString CXml::ToUTF8( CString & s, CString from_codec, bool nosub )
 {
 	int i;
 	CString dst = "",dst1;
@@ -221,7 +282,12 @@ CString CXml::ToUTF8( CString & s )
 	if ( s == "" )
 		return dst;
 
-	dst = xml_isolat1ToUTF8( s.Data(), s.Length() );
+	dst = xml_InputToUTF8( s.Data(), s.Length(), from_codec );
+	
+	if ( nosub == TRUE )
+	{
+		return dst;
+	}
 	
 	dst = dst.Replace("&","&amp;");
 	dst = dst.Replace("\'","&apos;");
@@ -253,7 +319,7 @@ CString CXml::ToUTF8( CString & s )
 }
 
 /** */
-CString CXml::FromUtf8( CString & s )
+CString CXml::FromUtf8( CString & s, CString to_codec, bool nosub )
 {
 	int i,i1;
 	CString c = "",c1 = "";
@@ -261,7 +327,12 @@ CString CXml::FromUtf8( CString & s )
 
 	if ( s != "" )
 	{
-		c = xml_UTF8Toisolat1( s.Data(), s.Length() );
+		c = xml_UTF8ToOutput( s.Data(), s.Length(), to_codec );
+	}
+	
+	if ( nosub == TRUE )
+	{
+		return c;
 	}
 
 	for(i=0;i<c.Length();i++)
