@@ -48,6 +48,8 @@
 #include <dclib/core/platform.h>
 #include <dclib/cfilemanager.h>
 
+#include <dclib/core/clogfile.h>
+
 #include "ctransfer.h"
 
 #define READ_FILE_BUFFER_SIZE	(40*1024)
@@ -343,7 +345,7 @@ void CTransfer::ConnectionState( eConnectionState state )
 }
 
 /** */
-int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglong endposition, ulonglong size, ulonglong chunksize, CString srcfile )
+int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglong endposition, ulonglong size, ulonglong chunksize, CString srcfile, CString adcTTH )
 {
 	if ( (srcfile == "" ) && (eMedium == eltFILE) )
 	{
@@ -363,6 +365,12 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 		return -1;
 	}
 
+	if (adcTTH.Left(4).ToUpper() == "TTH:")
+	{
+		printf("CTransfer::StartDownload Warning! Removed TTH: prefix from TTH\n");
+		adcTTH = adcTTH.Mid(4, adcTTH.Length() - 4);
+	}
+	
 	SetMode(estTRANSFERDOWNLOAD);
 	SetStartPosition(startposition);
 	SetEndPosition(endposition);
@@ -383,7 +391,16 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 	{
 		if ( (GetDstFilename() == DC_USER_FILELIST_HE3) && (m_MessageSupports.m_bXMLBZList == TRUE) )
 		{
-			SendGet( DC_USER_FILELIST_XMLBZ, startposition+1 );
+			if (m_MessageSupports.m_bADCGet == TRUE)
+			{
+				SendADCGet( "file", DC_USER_FILELIST_XMLBZ, 0, -1 );
+			}
+			else
+			{
+				// SendUGetBlock causes an infinite loop of filelist downloads! not anymore!
+				SendUGetBlock( DC_USER_FILELIST_XMLBZ, startposition );
+				// SendGet( DC_USER_FILELIST_XMLBZ, startposition+1 );
+			}
 		}
 		else if ( (GetDstFilename() == DC_USER_FILELIST_HE3) && (m_MessageSupports.m_bBZList == TRUE) )
 		{
@@ -391,13 +408,43 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 		}
 		else
 		{
-			if ( chunksize == 0 )
+			// if we don't have a TTH, we cannot do ADCGet
+			// we'll try to fallback to other get commands, even though these may not work with DC++ >= 0.696
+			if ( (m_MessageSupports.m_bADCGet == TRUE) && (m_MessageSupports.m_bTTHF == TRUE) && (adcTTH != "") )
+			{
+				if ( (m_MessageSupports.m_bZLIG) && (CConfig::Instance()->GetCompressedTransfers() == TRUE) )
+				{
+					SendADCGet( "file", "TTH/" + adcTTH, startposition, chunksize, TRUE );
+					m_nDataType = 1;
+				}
+				else
+				{
+					SendADCGet ( "file", "TTH/" + adcTTH, startposition, chunksize, FALSE );
+					m_nDataType = 0;
+				}
+			}
+			else if ( chunksize == 0 )
 			{
 				SendGet( GetDstFilename(), startposition+1 );
 				m_nDataType = 0;
 			}
+			else if (m_MessageSupports.m_bXMLBZList == TRUE)
+			{
+				if ( (m_MessageSupports.m_bZBlock == TRUE) &&
+				     (CConfig::Instance()->GetCompressedTransfers() == TRUE) )
+				{
+					SendUGetZBlock( GetDstFilename(), startposition, chunksize );
+					m_nDataType = 1;
+				}
+				else
+				{
+					SendUGetBlock( GetDstFilename(), startposition, chunksize);
+					m_nDataType = 0;
+				}
+			}
 			else if ( (m_MessageSupports.m_bChunk == FALSE) &&
-			          (m_MessageSupports.m_bZBlock == FALSE) )
+			          ( (m_MessageSupports.m_bZBlock == FALSE) ||
+				    (CConfig::Instance()->GetCompressedTransfers() == FALSE) ) )
 			{
 				SendGet( GetDstFilename(), startposition+1 );
 				m_nDataType = 0;
@@ -407,7 +454,8 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 				SendGet( GetDstFilename(), startposition+1, chunksize );
 				m_nDataType = 0;
 			}
-			else if ( m_MessageSupports.m_bZBlock == TRUE )
+			else if ( (m_MessageSupports.m_bZBlock == TRUE) &&
+			          (CConfig::Instance()->GetCompressedTransfers() == TRUE) )
 			{
 				SendGetZBlock( GetDstFilename(), startposition, chunksize );
 				m_nDataType = 1;
@@ -423,7 +471,7 @@ int CTransfer::StartDownload( CString dstfile, ulonglong startposition, ulonglon
 }
 
 /** */
-int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ulonglong chunksize, CString srcfile )
+int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ulonglong chunksize, CString srcfile, bool uget, bool adcget, CString adcTTH )
 {
 	if ( (srcfile == "" ) && (eMedium != eltBUFFER) )
 	{
@@ -460,8 +508,64 @@ int CTransfer::StartUpload( CString dstfile, ulonglong length, ulonglong pos, ul
 
 	/** set starttime */
 	InitTime();
-
-	SendFileLength(length);
+	
+	if (adcget)
+	{
+		if (GetMedium() == eltTTHL)
+		{
+			int treesize = prepareTTHLsend( adcTTH );
+			if (treesize == 0)
+			{
+				return -1;
+			}
+			else
+			{
+				SendADCSnd( "tthl", "TTH/" + adcTTH, 0, treesize);
+			}
+		}
+		else if (adcTTH == DC_USER_FILELIST_XMLBZ)
+		{
+			// damn hacks for correct response to filelist request
+			SendADCSnd( "file", adcTTH, pos, m_nChunkSize );
+		}
+		else if (adcTTH == "") // ADCGet without TTH
+		{
+			SendADCSnd( "file", "/" + dstfile, pos, m_nChunkSize );
+		}
+		else
+		{
+			SendADCSnd( "file", "TTH/" + adcTTH, pos, m_nChunkSize );
+		}
+		
+		if ( (eMode != estTRANSFERUPLOAD) || (m_bIdle == FALSE) )
+		{
+			DPRINTF("Warning: not in uploadmode/idle ADCSND [%d/%d]\n",eMode,m_bIdle);
+		}
+		else
+		{
+			DoInitUpload();
+		}
+	}
+	else if (uget)
+	{
+		SendSending(m_nChunkSize);
+		
+		// don't know why this does not work, instead code out has been copied out of HandleMessage()
+		//HandleMessage("$Send|", length);
+		
+		if ( (eMode != estTRANSFERUPLOAD) || (m_bIdle == FALSE) )
+		{
+			DPRINTF("Warning: not in uploadmode/idle SEND [%d/%d]\n",eMode,m_bIdle);
+		}
+		else
+		{
+			DoInitUpload();
+		}
+	}
+	else
+	{
+		SendFileLength(length);
+	}
 
 	return 0;
 }
@@ -488,7 +592,7 @@ bool CTransfer::DoInitDownload()
 
 		mode |= IO_RAW | IO_WRITEONLY;
 
-		if ( m_File.Open( sSrcFilename, mode, MO_IRUSR|MO_IWUSR|MO_IRGRP|MO_IROTH ) == FALSE )
+		if ( m_File.Open( sSrcFilename, mode, MO_IRUSR|MO_IWUSR|MO_IRGRP|MO_IWGRP|MO_IROTH|MO_IWOTH ) == FALSE )
 		{
 			CallBack_SendError(strerror(errno));
 			perror("File open");
@@ -575,7 +679,7 @@ int CTransfer::HandleMessage( char * c, int )
 				
 				if ( eMode != estTRANSFERDOWNLOAD || (m_bIdle == FALSE) )
 				{
-					DPRINTF("Warning: not in downloadmode/idle FILELENGTH/LISTLEN\n");
+					DPRINTF("Warning: not in downloadmode/idle SENDING\n");
 
 					delete Object;
 					Object = 0;
@@ -596,7 +700,11 @@ int CTransfer::HandleMessage( char * c, int )
 						m_nChunkSize = msg->m_nLength;
 					}
 
-					//m_nFileSize = msg->m_nFileLength;
+					// fixme why was this commented out?
+					if (m_nFileSize == 0)
+					{
+						m_nFileSize = msg->m_nLength;
+					}
 
 					// start transfer
 					if ( DoInitDownload() == TRUE )
@@ -609,6 +717,79 @@ int CTransfer::HandleMessage( char * c, int )
 				break;
 			}
 			
+			case DC_MESSAGE_ADCSND:
+			{
+				CMessageADCSnd * msg = (CMessageADCSnd*)Object;
+				if ( eMode != estTRANSFERDOWNLOAD || (m_bIdle == FALSE) )
+				{
+					DPRINTF("Warning: not in downloadmode/idle ADCSND\n");
+
+					delete Object;
+					Object = 0;
+				}
+				else
+				{
+					m_nTransfered = 0;
+					
+					if ( m_nEndPosition == 0 )
+					{
+						m_nEndPosition = msg->m_nSize;
+					}
+					
+					if ( m_nChunkSize != msg->m_nSize )
+					{
+						if ( m_nChunkSize == 0 )
+						{
+							m_nChunkSize = msg->m_nSize;
+						}
+						else
+						{
+							DPRINTF( "Error: wrong length: Requested=%llu Got=%lld\n", m_nChunkSize, msg->m_nSize );
+							CallBack_SendError("Remote client sent wrong data - incorrect length!");
+							Disconnect();
+							
+							delete Object;
+							Object = 0;
+							
+							break;
+						}
+					}
+					
+					if ( m_nStartPosition != msg->m_nPos )
+					{
+						DPRINTF( "Error: wrong start position: Requested=%llu Got=%llu\n", m_nStartPosition, msg->m_nPos );
+						CallBack_SendError("Remote client sent wrong data - incorrect start position!");
+						Disconnect();
+						
+						delete Object;
+						Object = 0;
+						
+						break;
+					}
+					
+					if (m_nFileSize == 0)
+					{
+						m_nFileSize = msg->m_nSize;
+					}
+					
+					// not sure if this is really necessary
+					if (msg->m_bZlib == TRUE)
+					{
+						m_nDataType = 1;
+					}
+					else
+					{
+						m_nDataType = 0;
+					}
+					
+					if ( DoInitDownload() == TRUE )
+					{
+						m_bIdle = FALSE;
+					}
+				}
+				
+				break;
+			}
 			case DC_MESSAGE_FILELENGTH:
 			case DC_MESSAGE_LISTLEN:
 			{
@@ -662,54 +843,7 @@ int CTransfer::HandleMessage( char * c, int )
 				}
 				else
 				{
-					m_nTransfered = 0;
-
-					// init filehandle
-					if ( eMedium == eltFILE )
-					{
-						m_nFileBufferPos    = READ_FILE_BUFFER_SIZE;
-						m_nFileBufferSize   = READ_FILE_BUFFER_SIZE;
-						m_nPendingSendBytes = 0;
-
-						pByteArray->SetSize(READ_FILE_BUFFER_SIZE);
-
-						m_File.Close();
-
-						if ( m_File.Open( sSrcFilename, IO_RAW | IO_READONLY ) == FALSE )
-						{
-							CallBack_SendError( CString("(File open) ") + strerror(errno) );
-							perror("File open");
-						}
-						else
-						{
-							// seek to the startposition
-							if ( m_File.Seek(m_nStartPosition,SEEK_SET) == FALSE )
-							{
-								CallBack_SendError( CString("(File seek) ") + strerror(errno) );
-								perror("File seek");
-
-								m_File.Close();
-							}
-						}
-
-						// check for error
-						if ( !m_File.IsOpen() )
-						{
-							Disconnect(TRUE);
-						}
-						else
-						{
-							// start transfer
-							m_bIdle = FALSE;
-						}
-					}
-					else
-					{
-						// start transfer
-						m_bIdle = FALSE;
-					}
-
-					DPRINTF("start upload ...'%s'\n",sSrcFilename.Data());
+					DoInitUpload();
 				}
 
 				break;
@@ -796,6 +930,16 @@ int CTransfer::HandleMessage( char * c, int )
 						s += "CHUNK ";
 						// minislots (files < 16k)
 						s += "MiniSlots ";
+						// ADCGet needed for compatibility with DC++ >= 0.696
+						s += "ADCGet ";
+						// TTHF needed for ADCGet
+						if ( CConfig::Instance()->GetDisableHashList() == FALSE )
+						{
+							s += "TTHF ";
+							
+							// TTHL support needed for future compatibility with DC++
+							s += "TTHL ";
+						}
 
 						if ( s != "" )
 						{
@@ -828,11 +972,14 @@ int CTransfer::HandleMessage( char * c, int )
 				m_MessageLock.m_nVersionMinor);
 				
 				// now we need to check some broken clients and limit support
+				// warning! 0.68 has minor version 68, not 680! - 
 				if ( (m_MessageLock.m_eClientVersion == eucvDCPP) &&
 				     (m_MessageLock.m_nVersionMajor == 0) &&
-				     (m_MessageLock.m_nVersionMinor <= 403) )
+				     ( (m_MessageLock.m_nVersionMinor >= 307) &&
+				       (m_MessageLock.m_nVersionMinor <= 403) ) )
 				{
 					// disable xml list for dc++ 0.401
+					printf("Warning! Disabling XML filelist support for DC++ between 0.307 and 0.403 inclusive.\n");
 					msg->m_bXMLBZList = FALSE;
 				}
 				
@@ -956,8 +1103,9 @@ void CTransfer::DataSend()
 		}
 
 		freebuf = READ_FILE_BUFFER_SIZE; //GetFreeSendBufferSize();
-
-		if ( (m_MessageSupports.m_bChunk == FALSE) || (m_nChunkSize == 0) )
+		
+		// should we test for zblock also?
+		if ( ((m_MessageSupports.m_bChunk == FALSE) && (m_MessageSupports.m_bADCGet == FALSE) && (m_MessageSupports.m_bXMLBZList == FALSE) && (m_MessageSupports.m_bZBlock == FALSE)) || (m_nChunkSize == 0) )
 		{
 			len = m_nFileSize - m_nStartPosition - m_nTransfered;
 		}
@@ -1008,7 +1156,7 @@ void CTransfer::DataSend()
 				len = m_nFileBufferSize-m_nFileBufferPos;
 			}
 		}
-		else if ( eMedium == eltBUFFER )
+		else if ( (eMedium == eltBUFFER) || (eMedium == eltTTHL) )
 		{
 			m_nFileBufferPos = m_nTransfered;
 		}
@@ -1077,8 +1225,13 @@ void CTransfer::DataSend()
 		}
 
 		if ( ((m_nStartPosition+m_nTransfered) == m_nFileSize) ||
-		     ( (m_MessageSupports.m_bChunk == TRUE) &&
-		       (m_nChunkSize == m_nTransfered) ) )
+		     ( ((m_MessageSupports.m_bChunk == TRUE) ||
+		        (m_MessageSupports.m_bADCGet == TRUE) ||
+			(m_MessageSupports.m_bXMLBZList == TRUE) ||
+			(m_MessageSupports.m_bZBlock == TRUE)) &&
+		       (m_nChunkSize == m_nTransfered)
+		     )
+		   )
 		{
 			DPRINTF("end found\n");
 			if ( eMedium == eltFILE )
@@ -1093,6 +1246,22 @@ void CTransfer::DataSend()
 			// stop transfer
 			m_bIdle = TRUE;
 
+			// write finished upload message into log file
+			if ( (CConfig::Instance()->GetLogFile() == TRUE) && (CConfig::Instance()->GetLogFinishedUploads() == TRUE) )
+			{
+				CString logmessage = "Upload done [" + sNick + "] ";
+				if (sSrcFilename == "")
+				{
+					logmessage += "filelist";
+				}
+				else
+				{
+					logmessage += sSrcFilename;
+				}
+				logmessage += " (" + CString().setNum(m_nTransfered) + "/" + CString().setNum(m_nFileSize) + ")";
+				CLogFile::Write(CConfig::Instance()->GetLogFileName(),eltINFO,logmessage);
+			}
+
 			// stop loop
 			lo = 5;
 		}
@@ -1157,7 +1326,7 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 				{
 					r = HandleFileTransfer(buffer+i,len-i);
 				}
-				else if ( eMedium == eltBUFFER )
+				else if ( (eMedium == eltBUFFER) || (eMedium == eltTTHL) )
 				{
 					r = HandleBufferTransfer(buffer+i,len-i);
 				}
@@ -1183,17 +1352,28 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 				if ( (m_nChunkSize == m_nTransfered) &&
 				    ((m_MessageSupports.m_bChunk == TRUE) ||
 				     (m_MessageSupports.m_bZBlock == TRUE) ||
+				     (m_MessageSupports.m_bXMLBZList == TRUE) ||
+				     (m_MessageSupports.m_bADCGet == TRUE) ||
 				     ((m_nStartPosition+m_nChunkSize) == m_nFileSize) ) )
 				{
 					m_bIdle = TRUE;
+					
+					// send transfer message does NOT go here, moving it here
+					// breaks multi download from original nmdc client microdc 0.11.0
+					// which is handled by a single $Send| getting multiple chunks
+				}
+				
+				// however, we can put the test m_nChunkSize == m_nTransfered, to
+				// avoid sending transfer messages which will just be ignored
+				if ( m_nChunkSize == m_nTransfered )
+				{
+					// send transfer message to handle new chunk-end
+					Object = new CMessageTransfer();
+					Object->m_eDirection  = edDOWNLOAD;
+					Object->m_nTransfered = m_nTransfered;
+					Object->m_nLength     = m_nFileSize;
+					CallBack_SendObject(Object);
 				}
-
-				// send transfer message to handle new chunk-end
-				Object = new CMessageTransfer();
-				Object->m_eDirection  = edDOWNLOAD;
-				Object->m_nTransfered = m_nTransfered;
-				Object->m_nLength     = m_nFileSize;
-				CallBack_SendObject(Object);
 
 				// check end
 				if ( m_nChunkSize == m_nTransfered )
@@ -1201,7 +1381,9 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 					// if endposition not filesize and remote don't support chunks we must disconnect
 					if ( ((m_nStartPosition+m_nChunkSize) != m_nFileSize) &&
 					     (m_MessageSupports.m_bChunk == FALSE) &&
-					     (m_MessageSupports.m_bZBlock == FALSE) )
+					     (m_MessageSupports.m_bZBlock == FALSE) &&
+					     (m_MessageSupports.m_bXMLBZList == FALSE) &&
+					     (m_MessageSupports.m_bADCGet == FALSE) )
 					{
 						Disconnect();
 					}
@@ -1214,12 +1396,47 @@ void CTransfer::DataAvailable( const char * buffer, int len )
 		}
 		else
 		{
-			r = HandleControlTransfer(buffer+i,len-i);
+			// Work around a bug caused by the size of compressed data being larger
+			// than the uncompressed data. This causes valknut to set the "idle" flag
+			// too early, when we still are getting compressed data. So we'll check that
+			// the data starts with "$", if it does then it's probably a command, otherwise
+			// it's probably data. No this is not a proper fix, just makes the bug
+			// much less likely to occur (since the compressed data might start with "$").
+			if (buffer[i] == '$')
+			{
+				r = HandleControlTransfer(buffer+i,len-i);
+				
+				// add traffic control
+				CSocket::m_Traffic.AddTraffic(ettCONTROLRX,r);
+			}
+			else
+			{
+				DPRINTF("Bug avoiding! Control transfer not starting with \"$\" is really data!\n");
+				if ( eMedium == eltFILE )
+				{
+					r = HandleFileTransfer(buffer+i,len-i);
+				}
+				else if ( (eMedium == eltBUFFER) || (eMedium == eltTTHL) )
+				{
+					r = HandleBufferTransfer(buffer+i,len-i);
+				}
+				else
+				{
+					r = -1;
+				}
+			}
 
-			// add traffic control
-			CSocket::m_Traffic.AddTraffic(ettCONTROLRX,r);
-
-			i += r;
+			if (r >= 0)
+			{
+				// add traffic control
+				CSocket::m_Traffic.AddTraffic(ettDATARX,r);
+				
+				i += r;
+			}
+			else
+			{
+				break;
+			}
 		}
 
 		if ( i == p )
@@ -1253,6 +1470,9 @@ int CTransfer::HandleFileTransfer( const char * buffer, int len )
 		inlen  = len;
 		if ( m_ZLib.InflateZBlock( (char*)buffer, &inlen, (char*)zbuffer.Data(), &outlen ) == FALSE )
 		{
+			DPRINTF("HandleFileTransfer: inflate failed!\n");
+			serr = "Zlib decompression failed";
+			
 			// close file
 			m_File.Close();
 
@@ -1351,6 +1571,9 @@ int CTransfer::HandleBufferTransfer( const char * buffer, int len )
 		inlen  = len;
 		if ( m_ZLib.InflateZBlock( (char*)buffer, &inlen, (char*)zbuffer.Data(), &outlen ) == FALSE )
 		{
+			DPRINTF("HandleBufferTransfer: inflate failed!\n");
+			serr = "Zlib decompression failed";
+			
 			CallBack_SendError(serr);
 
 			// disconnect on error
@@ -1496,3 +1719,77 @@ int CTransfer::HandleControlTransfer( const char * buffer, int len )
 
 	return p;
 }
+
+/** Code copied out of HandleMessage() so it can be called directly from another place */
+void CTransfer::DoInitUpload()
+{
+	m_nTransfered = 0;
+	// init filehandle
+	if ( eMedium == eltFILE )
+	{
+		m_nFileBufferPos    = READ_FILE_BUFFER_SIZE;
+		m_nFileBufferSize   = READ_FILE_BUFFER_SIZE;
+		m_nPendingSendBytes = 0;
+		pByteArray->SetSize(READ_FILE_BUFFER_SIZE);
+		m_File.Close();
+		
+		if ( m_File.Open( sSrcFilename, IO_RAW | IO_READONLY ) == FALSE )
+		{
+			CallBack_SendError( CString("(File open) ") + strerror(errno) );
+			perror("File open");
+		}
+		else
+		{
+			// seek to the startposition
+			if ( m_File.Seek(m_nStartPosition,SEEK_SET) == FALSE )
+			{
+				CallBack_SendError( CString("(File seek) ") + strerror(errno) );
+				perror("File seek");
+				m_File.Close();
+			}
+		}
+		// check for error
+		if ( !m_File.IsOpen() )
+		{
+			Disconnect(TRUE);
+		}
+		else
+		{
+			// start transfer
+			m_bIdle = FALSE;
+		}
+	}
+	else
+	{
+		// start transfer
+		m_bIdle = FALSE;
+	}
+	DPRINTF("start upload ...'%s' %lld/%lld\n",sSrcFilename.Data(),m_nStartPosition,m_nChunkSize);
+}
+
+/** */
+int CTransfer::prepareTTHLsend( CString tth )
+{
+	//DPRINTF("Get TTH leaves for %s\n", tth.Data() );
+	CByteArray * leaves = CFileManager::Instance()->GetHashLeaves( tth );
+	
+	if (leaves != 0)
+	{
+		SetBuffer(leaves);
+		
+		int size = leaves->Size();
+		SetLength(size);
+		m_nChunkSize = 0;
+		SetStartPosition(0);
+		SetEndPosition(size);
+		
+		return size;
+	}
+	else
+	{
+		DPRINTF("Hash leaves not found for %s\n", tth.Data());
+		SendError("File Not Available");
+		Disconnect();
+		return 0;
+	}
+}
