@@ -2,7 +2,7 @@
                           dcobject.h  -  description
                              -------------------
     begin                : Fri Feb 22 2002
-    copyright            : (C) 2002-2003 by Mathias Kster
+    copyright            : (C) 2002-2004 by Mathias Kster
     email                : mathen@users.berlios.de
  ***************************************************************************/
 
@@ -34,7 +34,8 @@
 #define DC_USER_FILELIST_HE3 	"MyList.DcLst"
 #define DC_USER_FILELIST 	DC_USER_FILELIST_HE3
 #define DC_USER_FILELIST_BZ 	"MyList.bz2"
-#define DC_USER_FILELIST_XML	"files.xml.bz2"
+#define DC_USER_FILELIST_XMLBZ	"files.xml.bz2"
+#define DC_USER_INDEXLIST	"index.lst"
 
 #ifdef WIN32
 template class DLL_EXPORT CList<CString>;
@@ -115,7 +116,16 @@ typedef enum eDCMessage {
 	// nmdc
 	DC_MESSAGE_GET_NET_INFO,
 	// user command
-	DC_MESSAGE_USER_COMMAND
+	DC_MESSAGE_USER_COMMAND,
+	// own ip
+	DC_MESSAGE_YOURIP,
+	// ZLIB compression
+	DC_MESSAGE_GETZBLOCK,
+	DC_MESSAGE_SENDING,
+	//DC_MESSAGE_FAILED
+	// ADCGET
+	DC_MESSAGE_ADCGET,
+	DC_MESSAGE_ADCSND
 };
 
 #define EDONKEY_HASH_BLOCK_SIZE 9728000
@@ -310,6 +320,23 @@ public:
 		m_bMiniSlots = FALSE;
 		m_bXMLBZList = FALSE;
 		m_bZBlock    = FALSE;
+		m_bADCGet    = FALSE;
+		m_bTTHL      = FALSE;
+		m_bTTHF      = FALSE;
+		m_bZLIG      = FALSE;
+		m_bUserCmd   = FALSE;
+		m_bNoGetInfo = FALSE;
+		m_bUserIP    = FALSE;
+		m_bUserIP2   = FALSE;
+		m_bXRules    = FALSE;
+		m_bBotINFO   = FALSE;
+		m_bNoHello   = FALSE;
+		m_bOpPlus    = FALSE;
+		m_bMCTo      = FALSE;
+		m_bBotList   = FALSE;
+		m_bClientID  = FALSE;
+		m_bSecExec   = FALSE;
+		m_bFeed      = FALSE;
 	};
 	/** */
 	virtual ~CMessageSupports() {};
@@ -319,15 +346,35 @@ public:
 	{
 		(CDCMessage&)*this = (CDCMessage&)p;
 
+		m_sContent   = p.m_sContent;
 		m_bBZList    = p.m_bBZList;
 		m_bSSL       = p.m_bSSL;
 		m_bChunk     = p.m_bChunk;
 		m_bMiniSlots = p.m_bMiniSlots;
 		m_bXMLBZList = p.m_bXMLBZList;
 		m_bZBlock    = p.m_bZBlock;
+		m_bADCGet    = p.m_bADCGet;
+		m_bTTHL      = p.m_bTTHL;
+		m_bTTHF      = p.m_bTTHF;
+		m_bZLIG      = p.m_bZLIG;
+		m_bUserCmd   = p.m_bUserCmd;
+		m_bNoGetInfo = p.m_bNoGetInfo;
+		m_bUserIP    = p.m_bUserIP;
+		m_bUserIP2   = p.m_bUserIP2;
+		m_bXRules    = p.m_bXRules;
+		m_bBotINFO   = p.m_bBotINFO;
+		m_bNoHello   = p.m_bNoHello;
+		m_bOpPlus    = p.m_bOpPlus;
+		m_bMCTo      = p.m_bMCTo;
+		m_bBotList   = p.m_bBotList;
+		m_bClientID  = p.m_bClientID;
+		m_bSecExec   = p.m_bSecExec;
+		m_bFeed      = p.m_bFeed;
 		return *this;
 	};
 
+	/** */
+	CString m_sContent;
 	/** */
 	bool m_bBZList;
 	/** */
@@ -340,6 +387,40 @@ public:
 	bool m_bXMLBZList;
 	/** */
 	bool m_bZBlock;
+	/** */
+	bool m_bADCGet;
+	/** */
+	bool m_bTTHL;
+	/** */
+	bool m_bTTHF;
+	/** */
+	bool m_bZLIG;
+	/** */
+	bool m_bUserCmd;
+	/** */
+	bool m_bNoGetInfo;
+	/** */
+	bool m_bUserIP;
+	/** */
+	bool m_bUserIP2;
+	/** */
+	bool m_bXRules;
+	/** */
+	bool m_bBotINFO;
+	/** */
+	bool m_bNoHello;
+	/** */
+	bool m_bOpPlus;
+	/** */
+	bool m_bMCTo;
+	/** */
+	bool m_bBotList;
+	/** */
+	bool m_bClientID;
+	/** */
+	bool m_bSecExec;
+	/** */
+	bool m_bFeed;
 };
 
 class DLL_EXPORT CMessageCapabilities : public CDCMessage {
@@ -400,6 +481,7 @@ public:
 	CMessageGet() : CDCMessage(DC_MESSAGE_GET) {
 		m_nPos  = 0;
 		m_nSize = 0;
+		m_bUGet = false;
 	};
 	/** */
 	virtual ~CMessageGet() {};
@@ -412,6 +494,7 @@ public:
 		m_sFilename = p.m_sFilename;
 		m_nPos      = p.m_nPos;
 		m_nSize     = p.m_nSize;
+		m_bUGet     = p.m_bUGet;
 
 		return *this;
 	};
@@ -422,6 +505,82 @@ public:
 	ulonglong m_nPos;
 	/** */
 	ulonglong m_nSize;
+	/** True if remote did $UGetBlock, false if $Get was used */
+	bool m_bUGet;
+};
+
+class DLL_EXPORT CMessageADCGet : public CDCMessage {
+public:
+	/** */
+	CMessageADCGet() : CDCMessage(DC_MESSAGE_ADCGET) {
+		m_nPos  = 0;
+		m_nSize = -1;
+		m_bZlib = false;
+	};
+	/** */
+	virtual ~CMessageADCGet() {};
+	
+	/** */
+	CMessageADCGet & operator = (const CMessageADCGet & p)
+	{
+		(CDCMessage&)*this = (CDCMessage&)p;
+		
+		m_sType = p.m_sType;
+		m_sTTH  = p.m_sTTH;
+		m_nPos  = p.m_nPos;
+		m_nSize = p.m_nSize;
+		m_bZlib = p.m_bZlib;
+		
+		return *this;
+	};
+	
+	/** file, tthl, or list (only file supported) */
+	CString m_sType;
+	/** */
+	CString m_sTTH;
+	/** */
+	ulonglong m_nPos;
+	/** */
+	long long m_nSize;
+	/** */
+	bool m_bZlib;
+};
+
+class DLL_EXPORT CMessageADCSnd : public CDCMessage {
+public:
+	/** */
+	CMessageADCSnd() : CDCMessage(DC_MESSAGE_ADCSND) {
+		m_nPos  = 0;
+		m_nSize = -1;
+		m_bZlib = false;
+	};
+	/** */
+	virtual ~CMessageADCSnd() {};
+	
+	/** */
+	CMessageADCSnd & operator = (const CMessageADCSnd & p)
+	{
+		(CDCMessage&)*this = (CDCMessage&)p;
+		
+		m_sType = p.m_sType;
+		m_sTTH  = p.m_sTTH;
+		m_nPos  = p.m_nPos;
+		m_nSize = p.m_nSize;
+		m_bZlib = p.m_bZlib;
+		
+		return *this;
+	};
+	
+	/** file, tthl, or list (only file supported) */
+	CString m_sType;
+	/** */
+	CString m_sTTH;
+	/** */
+	ulonglong m_nPos;
+	/** */
+	long long m_nSize;
+	/** */
+	bool m_bZlib;
 };
 
 class DLL_EXPORT CMessageGetPass : public CDCMessage {
@@ -676,6 +835,29 @@ public:
 	ulonglong m_nFileLength;
 };
 
+class DLL_EXPORT CMessageSending : public CDCMessage {
+public:
+	/** */
+	CMessageSending() : CDCMessage(DC_MESSAGE_SENDING) {
+		m_nLength = 0;
+	};
+	/** */
+	virtual ~CMessageSending() {};
+
+	/** */
+	CMessageSending & operator = (const CMessageSending & p)
+	{
+		(CDCMessage&)*this = (CDCMessage&)p;
+
+		m_nLength = p.m_nLength;
+
+		return *this;
+	};
+
+	/** */
+	ulonglong m_nLength;
+};
+
 class DLL_EXPORT CMessageDirection : public CDCMessage {
 public:
 	/** */
@@ -830,6 +1012,7 @@ class DLL_EXPORT CMessageMyInfo : public CDCMessage {
 public:
 	/** */
 	CMessageMyInfo() : CDCMessage(DC_MESSAGE_MYINFO) {
+		m_bValid         = FALSE;
 		m_nShared        = 0;
 		m_bOperator      = FALSE;
 		m_eAwayMode      = euamNONE;
@@ -848,6 +1031,7 @@ public:
 		(CDCMessage&)*this = (CDCMessage&)p;
 
 		m_sNick          = p.m_sNick;
+		m_bValid         = p.m_bValid;
 		m_sComment       = p.m_sComment;
 		m_sUnknown       = p.m_sUnknown;
 		m_eUserSpeed     = p.m_eUserSpeed;
@@ -870,6 +1054,8 @@ public:
 
 	/** the nickname */
 	CString m_sNick;
+	/** true if myinfo recv. */
+	bool m_bValid;
 	/** the comment */
 	CString m_sComment;
 	/** */
@@ -962,7 +1148,7 @@ public:
 		m_bLocal      = FALSE;
 		m_nPort       = 0;
 		m_bSizeLimit  = FALSE;
-		m_bSizeAtMost = FALSE;
+		m_eSizeType   = esstATLEAST;
 		m_nSize       = 0;
 		m_eFileType   = eftUNKNOWN;
 		m_bExtended   = FALSE;
@@ -979,7 +1165,7 @@ public:
 		m_sSource     = p.m_sSource;
 		m_nPort       = p.m_nPort;
 		m_bSizeLimit  = p.m_bSizeLimit;
-		m_bSizeAtMost = p.m_bSizeAtMost;
+		m_eSizeType   = p.m_eSizeType;
 		m_nSize       = p.m_nSize;
 		m_eFileType   = p.m_eFileType;
 		m_sString     = p.m_sString;
@@ -997,7 +1183,7 @@ public:
 	/** search options */
 	bool m_bSizeLimit;
 	/** search options */
-	bool m_bSizeAtMost;
+	enum eSearchSizeType m_eSizeType;
 	/** search options */
 	ulonglong m_nSize;
 	/** search options */
@@ -1058,6 +1244,7 @@ public:
 		m_sHash     = p.m_sHash;
 		m_nFreeSlot = p.m_nFreeSlot;
 		m_nMaxSlot  = p.m_nMaxSlot;
+		m_bFolder   = p.m_bFolder;
 
 		return *this;
 	};
@@ -1078,6 +1265,8 @@ public:
 	unsigned int m_nFreeSlot;
 	/** */
 	unsigned int m_nMaxSlot;
+	/** True if the result is a folder, false if the result is a file*/
+	bool m_bFolder;
 };
 
 class DLL_EXPORT CMessagePrivateChat : public CDCMessage {
